-- "Sea Traders"
-- By Tim Sandford, 2021
-- Comments to timsandford42@hotmail.com

require("file42")
require("math42")
require("music42")
require("table42")

function love.run()   -- Main function
  gVERSION, gPATCH, gREVISION, gCODENAME = love.getVersion()
  if gVERSION >=  11 then  -- Colour fix for 11.0 onwards
    local oldSetColor = love.graphics.setColor
    love.graphics.setColor = function (r, g, b, a)     
      if type(r)=="table" then
        r, g, b, a = r[1] / 255, r[2] / 255, r[3] / 255, (r[4] or 255) / 255
      else
        r, g, b, a = r / 255, g / 255, b / 255, (a or 255) / 255
      end
      oldSetColor(r,g,b,a)
    end
  end
  math.randomseed(os.time())  -- True randomness
  love.load(love.arg.parseGameArguments(arg), arg) -- load variables etc.
  while true do  -- Main loop
    if love.event then -- Process events
      love.event.pump()
      for name, a,b,c,d,e,f in love.event.poll() do
        if name == "quit" then
          if not love.quit or not love.quit() then
            return a
          end
        end
        love.handlers[name](a,b,c,d,e,f)
      end
    end
    love.update() -- Update
    love.graphics.origin() -- Not sure, probably reset origin?
    love.graphics.clear(love.graphics.getBackgroundColor()) -- Clears the screen
    love.draw() -- Collect draw commands
    love.graphics.present() -- Present the graphics on the screen!
    love.timer.sleep(0.001) -- This is to lower the load on the CPU, but I could edit it :)	
  end
end

function love.load(arg)  -- Load everything in
  local i, j
  io.stdout:setvbuf("no")                                         -- Print messages immediately, allows tracing bugs
  if arg[#arg] == "-debug" then require("mobdebug").start() end   -- Zerobrane debugging
  gSCREEN_W, gSCREEN_H = love.graphics.getWidth(), love.graphics.getHeight() -- Get screensize
  gRATIOX, gRATIOY = gSCREEN_W/800, gSCREEN_H/600                  -- Scale graphics to screen
  gFADE, gFADESPEED = 255, 5                                       -- Fade in effect
  gTIMER = love.timer.getTime()                                    -- Use timer in love.update for speed of game
  gFLASH, gFLASH_z, gFLASH_min, gFLASH_max = 255, 10, 80, 255		 -- Flashing images/text transparency
  mTRACK, gTRACK, gFADEIN, gFADEOUT = {}, 1, 0, 0                 -- Music tracks
  mTRACK[1], mTRACK[2], mBATTLE, mCOUNT = {}, {}, false, 0        -- Music tracks
  gBG_X, gBG_Y, gBG_Z, gBG_COUNT = 0, 0, 1, 0                     -- Background
  gTYPING, gTYPEDTEXT = false, ""                                  -- Used for text input
  gFLOATTEXT = {}                                                  -- Floating text on screen
  gENDING = 0

  love.mouse.setCursor(love.mouse.newCursor("gui/cursor.png", 32, 32))
  
  fTINY = love.graphics.newFont("fonts/regular.ttf", 16)          -- tiny font
  fSMALL = love.graphics.newFont("fonts/regular.ttf", 20)         -- small font
  fDEFAULT = love.graphics.newFont("fonts/regular.ttf", 24)       -- default font
  fSUBHEADING = love.graphics.newFont("fonts/heading.ttf", 32)    -- sub heading font
  fHEADING = love.graphics.newFont("fonts/heading.ttf", 64)       -- heading font
  fTITLE = love.graphics.newFont("fonts/heading.ttf", 100)        -- title font
  
  tPLACES = readsinglelist("data/islands.txt")
  tGOVERNMENT = readmultitable("data/governments.txt")
  tBLUEPRINT = readmultitable("data/ships.txt")
  tSHIPNAME = readsinglelist("data/shipnames.txt")
  tCAPNAME = readsinglelist("data/capnames.txt")
  tFIRST = readsinglelist("data/firstnames.txt")
  tMIDDLE = readsinglelist("data/nicknames.txt")
  tLAST = readsinglelist("data/surnames.txt")
  tROLES = readmultitable("data/crewroles.txt")
  tGOODS = readmultitable("data/goods.txt")
  tRANKS = readsinglelist("data/ranks.txt")
  tUPGRADES = readmultitable("data/upgrades.txt")
  
  for i = 1, #tBLUEPRINT, 1 do
    tBLUEPRINT[i].Image = {}
    for j = 1, 4, 1 do
      tBLUEPRINT[i].Image[j] = love.graphics.newImage("ships/ship"..tBLUEPRINT[i].Type..j..".png")
    end
    tBLUEPRINT[i].Image[5] = love.graphics.newImage("ships/ship99.png")
  end
  
  iICON, iBACKGROUND, sSOUND, iOCEAN, iISLAND, iEXPLODE, sWAVES = {}, {}, {}, {}, {}, {}, {}
  for i = 1, 99, 1 do
    if love.filesystem.getInfo("gui/icon"..i..".png", "file") then iICON[i] = love.graphics.newImage("gui/icon"..i..".png") end
    if love.filesystem.getInfo("sounds/sound"..i..".ogg", "file") then sSOUND[i] = love.audio.newSource("sounds/sound"..i..".ogg", "static") end
    if love.filesystem.getInfo("backgrounds/background"..i..".jpg", "file") then iBACKGROUND[i] = love.graphics.newImage("backgrounds/background"..i..".jpg") end
    if love.filesystem.getInfo("ocean/ocean"..i..".png", "file") then iOCEAN[i] = love.graphics.newImage("ocean/ocean"..i..".png") end
    if love.filesystem.getInfo("islands/isle_"..i..".png", "file") then iISLAND[i] = love.graphics.newImage("islands/isle_"..i..".png") end
    if love.filesystem.getInfo("effects/explode"..i..".png", "file") then iEXPLODE[i] = love.graphics.newImage("effects/explode"..i..".png") end
    if love.filesystem.getInfo("sounds/waves"..i..".ogg", "file") then sWAVES[i] = love.audio.newSource("sounds/waves"..i..".ogg", "static") end
  end
  iUNKNOWN = love.graphics.newImage("gui/unknown.png")
  iBOX = love.graphics.newImage("gui/box.png")
  iMAP = love.graphics.newImage("gui/parchment.png")
  iFRAME = love.graphics.newImage("gui/frame.png")
  iLANTERN = love.graphics.newImage("effects/lantern.png")
  sWIND = love.audio.newSource("sounds/wind.ogg", "static")
  
  if love.filesystem.getInfo("settings.txt", "file") then tSETTINGS = readsingletable("settings.txt")
  else tSETTINGS = readsingletable("data/defaultsettings.txt") end
  
  musicload(tSETTINGS.MusicVol)
  gOLDGAMEMODE = "menu"
  startmenu()
end

function savegame()   -- Save the game
  local buy = love.filesystem.newFile("save_buy.txt")
  local sell = love.filesystem.newFile("save_sell.txt")
  local i, j
  -- Save the player's ship
  pSHIP.X, pSHIP.Y = math.floor(pSHIP.X+0.5), math.floor(pSHIP.Y+0.5)
  writesingletable("save_player.txt", pSHIP)
  -- Save the player's cargo
  writesinglelist("save_cargo.txt", pSHIP.Cargo)
  -- Save the player's crew
  writemultitable("save_crew.txt", pSHIP.Crew)
  -- Save the player's upgrades
  writesinglelist("save_upgrades.txt", pSHIP.Upgrades)
  -- Save the comp ships
  if next(tSHIP)~=nil then
    for i = 1, #tSHIP, 1 do
      tSHIP[i].X, tSHIP[i].Y = math.floor(tSHIP[i].X+0.5), math.floor(tSHIP[i].Y+0.5)
      tSHIP[i].TX, tSHIP[i].TY = math.floor(tSHIP[i].TX+0.5), math.floor(tSHIP[i].TY+0.5)
      tSHIP[i].Upgrade = ""
      for j = 1, #tSHIP[i].Upgrades, 1 do
        if tSHIP[i].Upgrades[j]=="Yes" then tSHIP[i].Upgrade = tSHIP[i].Upgrade.."Y"
        else tSHIP[i].Upgrade = tSHIP[i].Upgrade.."N" end
      end
    end
    writemultitable("save_ships.txt", tSHIP)
  end
  -- Save the islands
  writemultitable("save_islands.txt", tISLAND)
  if love.filesystem.getInfo("save_buy.txt", "file") then love.filesystem.remove("save_buy.txt") end
  if love.filesystem.getInfo("save_sell.txt", "file") then love.filesystem.remove("save_sell.txt") end
  buy:open("w")
  sell:open("w")
  for i = 1, #tISLAND, 1 do
    for j = 1, #tGOODS, 1 do
      buy:write(tISLAND[i].Buy[j].."\r\n")
      sell:write(tISLAND[i].Sell[j].."\r\n")
    end
  end
  buy:close()
  sell:close()
  -- Save the wind & clock
  writesingletable("save_wind.txt", tWIND)
  writesingletable("save_clock.txt", tCLOCK)
  -- Save settings
  writesingletable("settings.txt", tSETTINGS)
end

function loadgame()   -- Load player game
  local i, j
  -- Load the player's ship
  pSHIP = readsingletable("save_player.txt")
  pSHIP.Image = {}
  for i = 1, 5, 1 do
    pSHIP.Image[i] = tBLUEPRINT[pSHIP.Type].Image[i]
  end
  gCAMERA_X, gCAMERA_Y = pSHIP.X, pSHIP.Y
  -- Load the player's cargo
  pSHIP.Cargo = readsinglelist("save_cargo.txt")
  -- Load the player's crew
  pSHIP.Crew = readmultitable("save_crew.txt")
  -- Load the player's upgrades
  pSHIP.Upgrades = readsinglelist("save_upgrades.txt")
  -- Load the comp ships
  tSHIP = readmultitable("save_ships.txt")
  if next(tSHIP)~=nil then
    for i = 1, #tSHIP, 1 do
      tSHIP[i].Crew = {}
      createcrewmember("Captain", i, "Ship", tSHIP[i].PlayerName)
      j = tSHIP[i].CrewNum
      while j>0 do
        j = j - 1
        createcrewmember("Random", i, "Ship")
      end
      tSHIP[i].Image = {}
      for j = 1, 5, 1 do
        tSHIP[i].Image[j] = tBLUEPRINT[tSHIP[i].Type].Image[j]
      end
      tSHIP[i].Upgrades = {}
      for j = 1, #tUPGRADES, 1 do
        if string.sub(tSHIP[i].Upgrade, j, j)=="Y" then tSHIP[i].Upgrades[j]="Yes"
        else tSHIP[i].Upgrades[j]="No" end
      end
    end
  end
  -- Load the islands
  tISLAND = readmultitable("save_islands.txt")
  for i = 1, #tISLAND, 1 do
    tISLAND[i].Menu = {"Yes", "Yes", "Yes", "Yes", "Yes", "Yes", "Yes"}
    if tISLAND[i].Government=="Uninhabited" then 
      tISLAND[i].Menu[2] = "No"
      tISLAND[i].Menu[3] = "No"
      tISLAND[i].Menu[4] = "No"
    elseif tISLAND[i].Government=="Indigenous Tribe" then 
      tISLAND[i].Menu[2] = "No" 
      tISLAND[i].Menu[4] = "No"
    else end
    tISLAND[i].Image = iISLAND[tISLAND[i].ImageRef]
    tISLAND[i].Data = love.image.newImageData("islands/isle_"..tISLAND[i].ImageRef..".png")
    tISLAND[i].Buy, tISLAND[i].Sell = {}, {}
  end
  if love.filesystem.getInfo("save_buy.txt", "file") then
    i, j = 1, 1
    for line in love.filesystem.lines("save_buy.txt") do
      tISLAND[i].Buy[j] = tonumber(line)
      j = j + 1
      if j>#tGOODS then
        i = i + 1
        j = 1
      end
    end
  end
  if love.filesystem.getInfo("save_sell.txt", "file") then
    i, j = 1, 1
    for line in love.filesystem.lines("save_sell.txt") do
      tISLAND[i].Sell[j] = tonumber(line)
      j = j + 1
      if j>#tGOODS then
        i = i + 1
        j = 1
      end
    end
  end
  calculatemaps()
  calculateimportsexports()
  distributecrew()
  -- Load wind & clock
  tWIND = readsingletable("save_wind.txt")
  tCLOCK = readsingletable("save_clock.txt")
  -- Prepare globals
  gGAMEMODE="shoreline"
  gFADE = 255
  playsound(20)
  love.mouse.setVisible(true)
  playwind("Off")
  playwaves("Off")
  if tSETTINGS.Music=="Yes" then musicplay(math.random(6)+5, "No") end
  tBULLET, tEXPLODE = {}, {}
  gENDING = 0
end

function startmenu()  -- Load the main menu
  gGAMEMODE = "menu"
  love.mouse.setVisible(true)
  if tSETTINGS.Music=="Yes" then musicplay(1) end
  gFADE = 255
end

function initialise()  -- Newgame, set things up (see also updatenewgame)
  local i
  tCLOCK, tWIND = {}, {}
  tCLOCK.Day = 1
  tCLOCK.Hour = 6
  tCLOCK.Min = 0
  tCLOCK.Count = tSETTINGS.FPS
  tWIND.Speed = 8
  tWIND.Start = tWIND.Speed
  tWIND.Direction = math.random(360)-1
  tWIND.Target = tWIND.Speed+math.random(16)-math.random(16)+math.random(16)-math.random(16)
  tISLAND, tSHIP = {}, {}
  tSETTINGS.MapZoom = 0.1
  tSETTINGS.Compass, tSETTINGS.Info, tSETTINGS.Wind, tSETTINGS.Names = "Yes", "Yes", "Yes", "No"
  createisland(0, 0, "Captain's Rest", 400, "Naval Court")
  tISLAND[1].Discovered = "Yes"
  createplayership()
  createcrewmember("Captain", 0, "Player Ship", pSHIP.PlayerName)
  createcrewmember("Navigator", 0, "Player Ship")
  tBULLET, tEXPLODE = {}, {}
  gENDING = 0
end

function spawnship(x, y, s, d)   -- Create an npc ship
  local ship = {}
  local i
  s = s or math.random(#tBLUEPRINT)
  d = d or math.random(100)
  ship.Name, ship.PlayerName = tSHIPNAME[math.random(#tSHIPNAME)], tCAPNAME[math.random(#tCAPNAME)]
  ship.Owner, ship.Type = "Comp"..s, s
  ship.Price, ship.Tag = tBLUEPRINT[s].Price, tBLUEPRINT[s].Tag
  ship.MaxSpeed, ship.MaxTurn = tBLUEPRINT[s].MaxSpeed, tBLUEPRINT[s].MaxTurn
  ship.Capacity, ship.Hull = tBLUEPRINT[s].Capacity, tBLUEPRINT[s].Hull
  ship.Range, ship.Power = tBLUEPRINT[s].Range, tBLUEPRINT[s].Power
  ship.Reload, ship.Difficulty = tBLUEPRINT[s].Reload, d
  ship.Upgrades, ship.Crew = {}, {}
  for i = 1, #tUPGRADES, 1 do
    if d>=tUPGRADES[i].Tech*7.5 then ship.Upgrades[i] = "Yes" else ship.Upgrades[i] = "No" end
  end
  ship.Image = {}
  for i = 1, 5, 1 do
    ship.Image[i] = tBLUEPRINT[s].Image[i]
  end
  ship.X, ship.Y, ship.Z = x, y, math.random(360)-1
  ship.TX, ship.TY, ship.Target = ship.X, ship.Y, 0
  ship.DX, ship.DY = ship.X, ship.Y
  ship.OX, ship.OY = ship.X, ship.Y
  ship.Resting, ship.Patrol = 0, 0
  ship.Behaviour = tBLUEPRINT[s].Default
  ship.Speed, ship.Ammo = 0, 100
  ship.Wounds, ship.Invulnerable = 0, 0
  ship.Firing, ship.Reloading, ship.Cannons = 0, 0, "None"
  ship.Anchor, ship.Sail, ship.Docked = "Up", "Full sail", 0
  ship.Width, ship.Height = ship.Image[1]:getWidth(), ship.Image[1]:getHeight()
  ship.Dying, ship.Delete = 0, "No"
  for i = 1, #tBLUEPRINT, 1 do
    ship["Attack"..i] = tBLUEPRINT[s]["Attack"..i]
  end
  ship["AttackP"] = tBLUEPRINT[s]["Attack"..pSHIP.Type]
  i = math.random(9)
  ship.CrewNum = i
  table.insert(tSHIP, deepcopy(ship))
  createcrewmember("Captain", #tSHIP, "Ship", ship.PlayerName)
  while i>0 do
    i = i - 1
    createcrewmember("Random", #tSHIP, "Ship")
  end
end

function getneartargetisland(ship)
  local distance, island = {}, {}
  local d, g, i, j
  -- Set all the distances to huge
  for i = 1, #tISLAND, 1 do
    distance[i], island[i] = math.huge, 0
  end
  -- Arrange the islands in order of distance from nearest to furthest
  for i = 1, #tISLAND, 1 do
    d = distancebetween(tISLAND[i].X, tISLAND[i].Y, ship.X, ship.Y)
    j = 1
    while distance[j]<d do
      j = j + 1
    end
    if island[j]==0 then
      table.remove(distance, j)
      table.remove(island, j)
    end
    table.insert(distance, j, d)
    table.insert(island, j, i)
  end
  -- Now look through the islands and choose a suitable target based upon ship type
  i, j = 0, 0
  repeat
    i = i + 1
    g = 1
    while tGOVERNMENT[g].Name~=tISLAND[island[i]].Government and g<#tGOVERNMENT do
      g = g + 1
    end
    if tGOVERNMENT[g].Name~=tISLAND[island[i]].Government then g = math.random(#tGOVERNMENT) end
    if island[i]~=ship.Origin and tGOVERNMENT[g]["Spawn"..ship.Type]>0 and math.random(100)>=70 then j=island[i] end
  until j~=0 or i==#tISLAND
  if i==#tISLAND then return math.random(#tISLAND)
  else return j end
end

function createplayership()  -- Create the player's ship
  local i
  pSHIP = {}
  purchaseship(1)
  pSHIP.Name, pSHIP.PlayerName, pSHIP.Owner = "Esmeralda", "Long John Silver", "Player"
  i = math.random(360)-1
  pSHIP.X, pSHIP.Y, pSHIP.Z = math.floor(math.cos(math.rad(bearingtoangle(i)))*325), math.floor(math.sin(math.rad(bearingtoangle(i)))*325), boundangle(i+90)
  pSHIP.TX, pSHIP.TY = nil, nil
  pSHIP.Speed, pSHIP.Gold, pSHIP.Food, pSHIP.Ammo = 0, 135, 50, 10
  pSHIP.Cargo, pSHIP.Wounds, pSHIP.Invulnerable = {}, 0, 0
  pSHIP.Firing, pSHIP.Reloading, pSHIP.Cannons = 0, 0, "None"
  pSHIP.Crew = {}
  pSHIP.Anchor, pSHIP.Sail, pSHIP.Docked = "Down", "Oars only", 0
  pSHIP.NavyRank, pSHIP.Cartographer, pSHIP.BountyHunter = -1, -1, -1
  pSHIP.Width, pSHIP.Height = pSHIP.Image[1]:getWidth(), pSHIP.Image[1]:getHeight()
  gCAMERA_X, gCAMERA_Y = pSHIP.X, pSHIP.Y
end

function purchaseship(s)
  local i
  if next(tBLUEPRINT[s])~=nil then
    pSHIP.Type = tBLUEPRINT[s].Type
    pSHIP.Price = tBLUEPRINT[s].Price
    pSHIP.Tag = tBLUEPRINT[s].Tag
    pSHIP.MaxSpeed = tBLUEPRINT[s].MaxSpeed
    pSHIP.MaxTurn = tBLUEPRINT[s].MaxTurn
    pSHIP.Capacity = tBLUEPRINT[s].Capacity
    pSHIP.Hull = tBLUEPRINT[s].Hull
    pSHIP.Range = tBLUEPRINT[s].Range
    pSHIP.Power = tBLUEPRINT[s].Power
    pSHIP.Reload = tBLUEPRINT[s].Reload
    pSHIP.Upgrades = {}
    for i = 1, #tUPGRADES, 1 do
      pSHIP.Upgrades[i] = "No"
    end
    pSHIP.Image = {}
    for i = 1, 5, 1 do
      pSHIP.Image[i] = tBLUEPRINT[s].Image[i]
    end
    if pSHIP.Cargo~=nil then 
      while gettablesize(pSHIP.Cargo)>tBLUEPRINT[s].Capacity do
        table.remove(pSHIP.Cargo)
      end
    end
  end
end

function createisland(x, y, name, size, govern)  -- Create an island
  local g, safety = 0, 0
  local i, j, n
  x, y = x or math.random(tSETTINGS.Mapsize*800)-(tSETTINGS.Mapsize*400), y or math.random(tSETTINGS.Mapsize*600)-(tSETTINGS.Mapsize*300)
  name = name or "Random"
  govern = govern or tGOVERNMENT[math.random(#tGOVERNMENT)].Name
  for i = 1, #tGOVERNMENT, 1 do
    if govern == tGOVERNMENT[i].Name then g = i end
  end
  n = gettablesize(tISLAND)+1
  if g~=0 then
    size = size or math.random(tGOVERNMENT[g].Maxsize-tGOVERNMENT[g].Minsize)+tGOVERNMENT[g].Minsize-1
    while not getislandlocationfree(x, y, size) and safety < 100 do
      safety = safety + 1
      x, y = math.random(tSETTINGS.Mapsize*800)-(tSETTINGS.Mapsize*400), math.random(tSETTINGS.Mapsize*600)-(tSETTINGS.Mapsize*300)
    end
    if getislandlocationfree(x, y, size) then
      tISLAND[n] = {}
      if name=="Random" then
        i = math.random(#tPLACES)
        tISLAND[n].Name = tPLACES[i]
        table.remove(tPLACES, i)
      else tISLAND[n].Name = name end 
      tISLAND[n].X, tISLAND[n].Y = x, y
      tISLAND[n].Size = size
      tISLAND[n].Government = govern
      tISLAND[n].Tag = tGOVERNMENT[g].Tag
      tISLAND[n].Wealth = tGOVERNMENT[g].Wealth + math.random(16) - math.random(16)
      tISLAND[n].Wealth = math.min(math.max(tISLAND[n].Wealth, 0), 100)
      tISLAND[n].Margin = math.random(20)+10
      tISLAND[n].Tech = math.max(tGOVERNMENT[g].Tech+math.random(2)-math.random(2), 0)
      tISLAND[n].Buy, tISLAND[n].Sell = {}, {}
      for j = 1, #tGOODS, 1 do
        if tGOVERNMENT[g]["Demand_"..tGOODS[j].Name]~=nil then 
          tISLAND[n].Buy[j] = math.floor((tGOODS[j].Value*tGOVERNMENT[g]["Demand_"..tGOODS[j].Name]*(0.8+(tISLAND[n].Wealth/250)))+0.5)+math.random(5)-math.random(5)
        else tISLAND[n].Buy[j] = 0 end
        tISLAND[n].Sell[j] = math.floor((tISLAND[n].Buy[j]*(100+tISLAND[n].Margin)/100)+0.5)
      end
      tISLAND[n].Population = math.ceil(size*tGOVERNMENT[g].Pop)
      tISLAND[n].Discovered = "No"
      i = math.random(#iISLAND)
      tISLAND[n].Image = iISLAND[i]
      tISLAND[n].ImageRef = i
      tISLAND[n].Data = love.image.newImageData("islands/isle_"..i..".png")
      tISLAND[n].Imports, tISLAND[n].Exports = "", ""
      tISLAND[n].Crew = {}
      tISLAND[n].Forage = 0
      tISLAND[n].Menu = {"Yes", "Yes", "Yes", "Yes", "Yes", "Yes", "Yes"}
      if tISLAND[n].Government=="Uninhabited" then 
        tISLAND[n].Menu[2] = "No"
        tISLAND[n].Menu[3] = "No"
        tISLAND[n].Menu[4] = "No"
      elseif tISLAND[n].Government=="Indigenous Tribe" then 
        tISLAND[n].Menu[2] = "No" 
        tISLAND[n].Menu[4] = "No"
      else end
    end
  end
end

function createcrewmember(role, dest, dest_type, name)  -- Create a crew member
  local crew, tot = {}, 0
  local i, j
  role, dest_type, name = role or "Random", dest_type or "Island", name or "Random"
  if role=="Random" then
    for i = 1, #tROLES, 1 do
      tot = tot + tROLES[i].Droprate
    end
    j = math.random(tot)
    tot = 0
    for i = 1, #tROLES, 1 do
      if j>tot and j<=tot+tROLES[i].Droprate then role = tROLES[i].Name end
      tot = tot + tROLES[i].Droprate
    end
  end
  for i = 1, #tROLES, 1 do
    if tROLES[i].Name==role then crew = deepcopy(tROLES[i]) end
  end
  if next(crew)~=nil then
    crew.Role = crew.Name
    if name~="Random" then crew.Name = name
    else crew.Name = tFIRST[math.random(#tFIRST)].." '"..tMIDDLE[math.random(#tMIDDLE)].."' "..tLAST[math.random(#tLAST)] end
    crew.Health = 70+math.random(30)
    crew.Morale = 70+math.random(30)
    crew.Days = 0
    if dest_type=="Island" then table.insert(tISLAND[dest].Crew, deepcopy(crew))
    elseif dest_type=="Player Ship" then table.insert(pSHIP.Crew, deepcopy(crew))      
    else table.insert(tSHIP[dest].Crew, deepcopy(crew)) end
  end
end

function calculateimportsexports()  -- Calculate imports and exports for all islands
  local imp_avg, exp_avg = {}, {}
  local imports, exports = {}, {}
  local imp_count, exp_count = {}, {}
  local i, j, k
  for j = 1, #tGOODS, 1 do
    imp_avg[j], exp_avg[j] = 0, 0
    imp_count[j], exp_count[j] = 0, 0
  end
  for i = 1, #tISLAND, 1 do
    for j = 1, #tGOODS, 1 do
      if tISLAND[i].Buy[j]~=0 then 
        imp_avg[j] = imp_avg[j] + tISLAND[i].Buy[j]
        imp_count[j] = imp_count[j] + 1
      end
      if tISLAND[i].Sell[j]~=0 then
        exp_avg[j] = exp_avg[j] + tISLAND[i].Sell[j]
        exp_count[j] = exp_count[j] + 1
      end
    end
  end
  for j = 1, #tGOODS, 1 do
    imp_avg[j] = imp_avg[j] / imp_count[j]
    exp_avg[j] = exp_avg[j] / exp_count[j]
  end
  for i = 1, #tISLAND, 1 do
    tISLAND[i].Imports, tISLAND[i].Exports = {}, {}
    imp_count, exp_count = {}, {}
    imports, exports = {}, {}
    for j = 1, #tGOODS, 1 do
      if tISLAND[i].Buy[j]~=0 and tISLAND[i].Buy[j]>=imp_avg[j] then
        k = 1
        while imp_count[k]~=nil and imp_count[k]>tISLAND[i].Buy[j]/imp_avg[j] do
          k = k + 1
        end
        table.insert(imports, k, tGOODS[j].Name)
        table.insert(imp_count, k, tISLAND[i].Buy[j]/imp_avg[j])
      end
      if tISLAND[i].Sell[j]~=0 and tISLAND[i].Sell[j]<=exp_avg[j] then
        k = 1
        while exp_count[k]~=nil and exp_count[k]<tISLAND[i].Sell[j]/exp_avg[j] do
          k = k + 1
        end
        table.insert(exports, k, tGOODS[j].Name)
        table.insert(exp_count, k, tISLAND[i].Sell[j]/exp_avg[j])
      end
    end
    for j = 1, 3, 1 do
      if imports[j]~=nil then 
        if j==1 then tISLAND[i].Imports = imports[j]
        else tISLAND[i].Imports = tISLAND[i].Imports.."\n"..imports[j] end
      end
      if exports[j]~=nil then 
        if j==1 then tISLAND[i].Exports = exports[j]
        else tISLAND[i].Exports = tISLAND[i].Exports.."\n"..exports[j] end
      end
    end
  end
end

function discoverfirstislands()
  local i, j, best, best_ref
  if tSETTINGS.StartIslands>=2 then   -- Start with a couple of nearby islands already discovered
    for j = 1, tSETTINGS.StartIslands-1, 1 do
      best, best_ref = math.huge, 0
      for i = 2, #tISLAND, 1 do
        if tISLAND[i].Discovered == "No" and distancebetween(tISLAND[1].X, tISLAND[1].Y, tISLAND[i].X, tISLAND[i].Y)<best then
          best = distancebetween(tISLAND[1].X, tISLAND[1].Y, tISLAND[i].X, tISLAND[i].Y)
          best_ref = i
        end
      end
      if best_ref~=0 then tISLAND[best_ref].Discovered = "Yes" end
    end
  end
end

function calculatemaps()  -- Distribute across the islands
  local i, j, maps, cost, distance, best, best_ref, entry
  local selected = {}
  for i = 1, #tISLAND, 1 do
    tISLAND[i].Maps = {}
    if tISLAND[i].Government=="Naval Court" then maps, cost = 4, 50
    elseif tISLAND[i].Government=="Governor" then maps, cost = 3, 40
    elseif tISLAND[i].Government=="Merchant Prince" then maps, cost = 2, 60
    elseif tISLAND[i].Government=="Settlement" then maps, cost = 1, 25
    elseif tISLAND[i].Government=="Pirate's Code" then maps, cost = 3, 30
    else maps, cost = 0, 0 end
    for j = 1, #tISLAND, 1 do
      selected[j] = false
    end
    while maps > 0 do
      best, best_ref = math.huge, 0
      maps = maps - 1
      for j = 1, #tISLAND, 1 do
        if i~=j and selected[j]==false and tISLAND[j].Discovered=="No" and distancebetween(tISLAND[i].X, tISLAND[i].Y, tISLAND[j].X, tISLAND[j].Y)<best then
          best = distancebetween(tISLAND[i].X, tISLAND[i].Y, tISLAND[j].X, tISLAND[j].Y)
          best_ref = j
        end
      end
      if best_ref ~= 0 then
        entry = {Name=tISLAND[best_ref].Name, Cost=cost, Index=best_ref}
        table.insert(tISLAND[i].Maps, deepcopy(entry))
        selected[best_ref] = true
      end
    end
  end
end

function distributecrew()  -- Distribute crew members across islands
  local i, c
  for i = 1, #tISLAND, 1 do
    tISLAND[i].Crew = {}
    if tISLAND[i].Government=="Pirate's Code" then c = math.random(5) - 1
    elseif tISLAND[i].Government=="Settlement" then c = math.random(2) - 1
    elseif tISLAND[i].Government=="Merchant Price" then c = math.random(4) - 1
    elseif tISLAND[i].Government=="Governor" then c = math.random(3) - 1
    elseif tISLAND[i].Government=="Naval Court" then c = math.random(5) - 1
    else c = 0 end
    while c>0 do
      createcrewmember("Random", i, "Island")
      c = c - 1
    end
  end
end

function setsail()  -- Set sail from the island
  love.mouse.setVisible(false)
  pSHIP.Docked = 0
  playsound(1)
  if tSETTINGS.Music=="Yes" then musicplay(math.random(6)+5, "No") end
  playwind("On")
  playwaves("On")
  gGAMEMODE = "sail"
  gFADE = 255
end

function love.update()  -- Runs every frame
  updategeneral()
  if gGAMEMODE == "sail" then 
    updategame()
    updateplayer()
    updateships()
    updatebullets()
    updateeffects()
  elseif gGAMEMODE == "newgame" then
    updatenewgame()
  else end
  musicupdate()
  collectgarbage()
  if love.timer.getTime()-gTIMER<=(1/tSETTINGS.FPS) then love.timer.sleep((1/tSETTINGS.FPS)-(love.timer.getTime()-gTIMER)) end  -- frame rate
  gTIMER = love.timer.getTime()  -- new timer
end

function updatenewgame()  -- Updates during world creation
  gSELECT = gSELECT + 1
  if gSELECT <= tSETTINGS.Islands then createisland()
  else
    print("Placed "..#tISLAND.." out of "..tSETTINGS.Islands.." islands.")
    calculateimportsexports()
    calculatemaps()
    discoverfirstislands()
    distributecrew()
    setsail()
    addfloatytext("Welcome aboard Captain "..pSHIP.PlayerName.."!\nPress 'H' for help.", tSETTINGS.FPS*8)
  end
end

function updategeneral()  -- Updates general variables, fade and flash
  if gFADE > 0 then  -- fade in graphics
    gFADE = gFADE - gFADESPEED
    if gFADE < 0 then gFADE = 0 end
  end
  gFLASH = gFLASH + gFLASH_z  -- flash counter
  if gFLASH > 255 then  -- this affects everything that flashes (same speed)
    gFLASH = gFLASH_max
    gFLASH_z = 0 - gFLASH_z
  elseif gFLASH < gFLASH_min then
    gFLASH = gFLASH_min
    gFLASH_z = 0 - gFLASH_z
  else end
  if next(gFLOATTEXT)~=nil and gGAMEMODE ~= "help" then
    for i = #gFLOATTEXT, 1, -1 do
      gFLOATTEXT[i].Count = gFLOATTEXT[i].Count - 1
      if gFLOATTEXT[i].Count <= 0 then table.remove(gFLOATTEXT, i) end
    end
  end
  if gENDING > 0 then
    gENDING = gENDING - 1
    if gENDING <= 0 then gGAMEMODE = "gameover" end
  end
end

function updategame()     -- Updates that only happen whilst sailing
  local i
  gBG_COUNT = gBG_COUNT + 1     -- Ocean animation
  if gBG_COUNT >= 9-math.sqrt(tWIND.Speed) then
    gBG_COUNT = 0
    gBG_Z = gBG_Z + 1
    if gBG_Z > #iOCEAN then gBG_Z = 1 end
  end
  tCLOCK.Count = tCLOCK.Count - tSETTINGS.ClockTime
  if tCLOCK.Count <= 0 then
    tCLOCK.Count = tSETTINGS.FPS
    tCLOCK.Min = tCLOCK.Min + 1
    tWIND.Direction = tWIND.Direction + math.random(3) - math.random(3)
    tWIND.Direction = boundangle(tWIND.Direction)
    tWIND.Speed = tWIND.Speed + ((tWIND.Target - tWIND.Start)/720)
    playwind()
    playwaves()
    if tCLOCK.Hour>=5 and tCLOCK.Hour<=19 and math.random(30)==1 and tWIND.Speed<20 then playsound(22)
    elseif tWIND.Speed>=32 and math.random(math.ceil(66-tWIND.Speed))==1 then playsound(23)
    else end
    if tCLOCK.Min >= 60 then processhour() end
  end
  if mCOUNT > 0 then 
    mCOUNT = math.max(mCOUNT - 1, 0)
    if mCOUNT == 0 then
      mBATTLE = false
      if tSETTINGS.Music=="Yes" then musicplay(5+math.random(6)) end
    end
  end
end

function updateplayer()   -- Updates to player variables including steering etc.
  local h = gethealth(pSHIP)
  local i
  --Steering
  if h>0 then
    if love.keyboard.isDown("left") and pSHIP.Anchor~="Down" and not love.keyboard.isDown("right") then
      if pSHIP.Sail=="Oars only" then pSHIP.Z = boundangle(pSHIP.Z - getmaxturn(pSHIP))
      elseif pSHIP.Sail=="Half sail" then pSHIP.Z = boundangle(pSHIP.Z - (getmaxturn(pSHIP)/2))
      elseif pSHIP.Sail=="Full sail" then pSHIP.Z = boundangle(pSHIP.Z - (getmaxturn(pSHIP)/4))
      else end
    elseif love.keyboard.isDown("right") and pSHIP.Anchor~="Down" and not love.keyboard.isDown("left") then
      if pSHIP.Sail=="Oars only" then pSHIP.Z = boundangle(pSHIP.Z + getmaxturn(pSHIP))
      elseif pSHIP.Sail=="Half sail" then pSHIP.Z = boundangle(pSHIP.Z + (getmaxturn(pSHIP)/2))
      elseif pSHIP.Sail=="Full sail" then pSHIP.Z = boundangle(pSHIP.Z + (getmaxturn(pSHIP)/4))
      else end
    else end
  end
  --Adjust speed & apply wind
  if pSHIP.Anchor=="Down" then 
    pSHIP.Speed = math.max(pSHIP.Speed-0.02, 0)
  elseif pSHIP.Sail=="Oars only" then 
    pSHIP.Speed = math.min(pSHIP.Speed+0.005, getmaxspeed(pSHIP)*0.2*getrowstrength(pSHIP))
    pSHIP.X = pSHIP.X + (math.cos(math.rad(bearingtoangle(tWIND.Direction)))*tWIND.Speed*0.004)
    pSHIP.Y = pSHIP.Y + (math.sin(math.rad(bearingtoangle(tWIND.Direction)))*tWIND.Speed*0.004)
    gBG_X = gBG_X - (math.cos(math.rad(bearingtoangle(tWIND.Direction)))*tWIND.Speed*0.004)
    gBG_Y = gBG_Y + (math.sin(math.rad(bearingtoangle(tWIND.Direction)))*tWIND.Speed*0.004)
  elseif pSHIP.Sail=="Half sail" then 
    pSHIP.Speed = math.min(pSHIP.Speed+0.01, getmaxspeed(pSHIP)*0.5)
    pSHIP.X = pSHIP.X + (math.cos(math.rad(bearingtoangle(tWIND.Direction)))*tWIND.Speed*0.016)
    pSHIP.Y = pSHIP.Y + (math.sin(math.rad(bearingtoangle(tWIND.Direction)))*tWIND.Speed*0.016)
    gBG_X = gBG_X - (math.cos(math.rad(bearingtoangle(tWIND.Direction)))*tWIND.Speed*0.016)
    gBG_Y = gBG_Y + (math.sin(math.rad(bearingtoangle(tWIND.Direction)))*tWIND.Speed*0.016)
  elseif pSHIP.Sail=="Full sail" then 
    pSHIP.Speed = math.min(pSHIP.Speed+0.02, getmaxspeed(pSHIP))
    pSHIP.X = pSHIP.X + (math.cos(math.rad(bearingtoangle(tWIND.Direction)))*tWIND.Speed*0.04)
    pSHIP.Y = pSHIP.Y + (math.sin(math.rad(bearingtoangle(tWIND.Direction)))*tWIND.Speed*0.04)
    gBG_X = gBG_X - (math.cos(math.rad(bearingtoangle(tWIND.Direction)))*tWIND.Speed*0.04)
    gBG_Y = gBG_Y + (math.sin(math.rad(bearingtoangle(tWIND.Direction)))*tWIND.Speed*0.04)
  else end
  --Move ship according to steering (& wind)
  pSHIP.X = pSHIP.X + (math.cos(math.rad(bearingtoangle(pSHIP.Z)))*pSHIP.Speed)
  pSHIP.Y = pSHIP.Y + (math.sin(math.rad(bearingtoangle(pSHIP.Z)))*pSHIP.Speed)
  gBG_X = gBG_X - (math.cos(math.rad(bearingtoangle(pSHIP.Z)))*pSHIP.Speed)
  gBG_Y = gBG_Y + (math.sin(math.rad(bearingtoangle(pSHIP.Z)))*pSHIP.Speed)
  gCAMERA_X, gCAMERA_Y = pSHIP.X, pSHIP.Y
  tilebackground()
  -- Player invulnerability count
  pSHIP.Invulnerable = math.max(pSHIP.Invulnerable-1, 0)
  -- Player firing weapon
  if pSHIP.Firing > 0 then -- Fire cannon balls
    pSHIP.Firing = math.max(pSHIP.Firing-1, 0)
    if pSHIP.Firing == 0 then
      pSHIP.Reloading = math.max(pSHIP.Reload-getreloadspeed(pSHIP), 50)
      i = 90+(math.random(getcannonaccuracy(pSHIP))-1)-(math.random(getcannonaccuracy(pSHIP))-1)
      if pSHIP.Cannons=="Right" then spawnbullet(pSHIP.X+(math.cos(math.rad(bearingtoangle(pSHIP.Z+i)))*(pSHIP.Width/2)), pSHIP.Y+(math.sin(math.rad(bearingtoangle(pSHIP.Z+i)))*(pSHIP.Width/2)), boundangle(pSHIP.Z+i), getcannonpower(pSHIP), getcannonrange(pSHIP), "Player")
      elseif pSHIP.Cannons=="Left" then spawnbullet(pSHIP.X+(math.cos(math.rad(bearingtoangle(pSHIP.Z-i)))*(pSHIP.Width/2)), pSHIP.Y+(math.sin(math.rad(bearingtoangle(pSHIP.Z-i)))*(pSHIP.Width/2)), boundangle(pSHIP.Z-i), getcannonpower(pSHIP), getcannonrange(pSHIP), "Player")
      else end  
      pSHIP.Ammo = pSHIP.Ammo - 1
      enterbattle()
      playsound(15)
    end
  end
  if pSHIP.Reloading > 0 then pSHIP.Reloading = math.max(pSHIP.Reloading - 1, 0) end
  -- Check for collisions with islands
  if h>0 then
    for i = 1, #tISLAND, 1 do
      if distancebetween(pSHIP.X, pSHIP.Y, tISLAND[i].X, tISLAND[i].Y)<=math.sqrt(2*((tISLAND[i].Size/2)^2)) then
        if checkislandcollision(pSHIP, tISLAND[i]) == true then collideisland(pSHIP) end
      end
    end
    if next(tSHIP)~=nil then
      for i = 1, #tSHIP, 1 do
        if distancebetween(pSHIP.X, pSHIP.Y, tSHIP[i].X, tSHIP[i].Y)<=(pSHIP.Height/2)+(tSHIP[i].Height/2) then
          if checkshipcollision(pSHIP, tSHIP[i]) == true then collideship(pSHIP) end
        end
      end
    end
  end
end

function updateships()    -- Updates to other ships
  local g, i, j, tx, ty
  local engine, turn, avoiding
  -- Spawn new ships if space
  updatespawnships()
  -- Loop through all current ships
  if next(tSHIP)~=nil then
    for i = #tSHIP, 1, -1 do
      --Adjust speed & apply wind
      if tSHIP[i].Anchor=="Down" then 
        tSHIP[i].Speed = math.max(tSHIP[i].Speed-0.02, 0)
      elseif tSHIP[i].Sail=="Oars only" then 
        tSHIP[i].Speed = math.min(tSHIP[i].Speed+0.005, getmaxspeed(tSHIP[i])*0.2*getrowstrength(tSHIP[i]))
        tSHIP[i].X = tSHIP[i].X + (math.cos(math.rad(bearingtoangle(tWIND.Direction)))*tWIND.Speed*0.004)
        tSHIP[i].Y = tSHIP[i].Y + (math.sin(math.rad(bearingtoangle(tWIND.Direction)))*tWIND.Speed*0.004)
      elseif tSHIP[i].Sail=="Half sail" then 
        tSHIP[i].Speed = math.min(tSHIP[i].Speed+0.01, getmaxspeed(tSHIP[i])*0.5)
        tSHIP[i].X = tSHIP[i].X + (math.cos(math.rad(bearingtoangle(tWIND.Direction)))*tWIND.Speed*0.016)
        tSHIP[i].Y = tSHIP[i].Y + (math.sin(math.rad(bearingtoangle(tWIND.Direction)))*tWIND.Speed*0.016)
      elseif tSHIP[i].Sail=="Full sail" then 
        tSHIP[i].Speed = math.min(tSHIP[i].Speed+0.02, getmaxspeed(pSHIP))
        tSHIP[i].X = tSHIP[i].X + (math.cos(math.rad(bearingtoangle(tWIND.Direction)))*tWIND.Speed*0.04)
        tSHIP[i].Y = tSHIP[i].Y + (math.sin(math.rad(bearingtoangle(tWIND.Direction)))*tWIND.Speed*0.04)
      else end
      -- Move ship according to steering
      tSHIP[i].X = tSHIP[i].X + (math.cos(math.rad(bearingtoangle(tSHIP[i].Z)))*tSHIP[i].Speed)
      tSHIP[i].Y = tSHIP[i].Y + (math.sin(math.rad(bearingtoangle(tSHIP[i].Z)))*tSHIP[i].Speed)
      -- Reduce invulnerability count
      tSHIP[i].Invulnerable = math.max(tSHIP[i].Invulnerable-1, 0)
      -- Enemy firing weapon
      if tSHIP[i].Firing > 0 then 
        tSHIP[i].Firing = math.max(pSHIP.Firing-1, 0)
        if tSHIP[i].Firing == 0 then
          tSHIP[i].Reloading = math.max(tSHIP[i].Reload-getreloadspeed(tSHIP[i]), 50)
          j = 90+(math.random(getcannonaccuracy(tSHIP[i]))-1)-(math.random(getcannonaccuracy(tSHIP[i]))-1)
          if tSHIP[i].Cannons=="Right" then spawnbullet(tSHIP[i].X+(math.cos(math.rad(bearingtoangle(tSHIP[i].Z+j)))*(tSHIP[i].Width/2)), tSHIP[i].Y+(math.sin(math.rad(bearingtoangle(tSHIP[i].Z+j)))*(tSHIP[i].Width/2)), boundangle(tSHIP[i].Z+j), getcannonpower(tSHIP[i]), getcannonrange(tSHIP[i]), "Ship"..tSHIP[i].Type)
          elseif tSHIP[i].Cannons=="Left" then spawnbullet(tSHIP[i].X+(math.cos(math.rad(bearingtoangle(tSHIP[i].Z-j)))*(tSHIP[i].Width/2)), tSHIP[i].Y+(math.sin(math.rad(bearingtoangle(tSHIP[i].Z-j)))*(tSHIP[i].Width/2)), boundangle(tSHIP[i].Z-j), getcannonpower(tSHIP[i]), getcannonrange(tSHIP[i]), "Ship"..tSHIP[i].Type)
          else end  
          tSHIP[i].Ammo = tSHIP[i].Ammo - 1
          if getisonscreen(tSHIP[i]) then playsound(15) end
        end
      end
      if tSHIP[i].Reloading > 0 then tSHIP[i].Reloading = math.max(tSHIP[i].Reloading - 1, 0) end
      -- Movement decisions here
      if gethealth(tSHIP[i])>0 then
        engine = 2        -- 2=Full sail, 1=Half sail, 0=Oars only
        -- These decisions to move based purely on behaviour
        if tSHIP[i].Behaviour == "Inbound" then
          -- Aim for my destination spot
          tSHIP[i].TX, tSHIP[i].TY = tSHIP[i].DX, tSHIP[i].DY
        elseif tSHIP[i].Behaviour == "Docked" then
          -- If I'm docked, wait until it's time to go then orbit
          tSHIP[i].Resting = tSHIP[i].Resting - 1
          if tSHIP[i].Resting == 0 then 
            engine = math.min(engine, 1)
            tSHIP[i].Anchor = "Up"
            tSHIP[i].Docked = 0
            if getisonscreen(tSHIP[i]) then playsound(3, "Yes") end
            j = getneartargetisland(tSHIP[i])
            tSHIP[i].Dest = j
            tSHIP[i].DX, tSHIP[i].DY = tISLAND[j].X, tISLAND[j].Y
            tSHIP[i].Behaviour = "Orbit"
          end
        elseif tSHIP[i].Behaviour == "Orbit" then
          -- If I'm orbiting, go out to beyond 200 then go around to my destination angle, then inbound to new island
          g = distancebetween(tSHIP[i].X, tSHIP[i].Y, tSHIP[i].OX, tSHIP[i].OY)
          if g<=(tISLAND[tSHIP[i].Origin].Size/2)+200 then
            j = getbearingbetween(tSHIP[i].OX, tSHIP[i].OY, tSHIP[i].X, tSHIP[i].Y)
            tSHIP[i].TX = tSHIP[i].OX + (math.cos(math.rad(bearingtoangle(j)))*((tISLAND[tSHIP[i].Origin].Size/2)+225))
            tSHIP[i].TY = tSHIP[i].OY + (math.sin(math.rad(bearingtoangle(j)))*((tISLAND[tSHIP[i].Origin].Size/2)+225))
          else
            -- Have I orbitted around to within 10degrees?
            j = getbearingbetween(tSHIP[i].OX, tSHIP[i].OY, tSHIP[i].DX, tSHIP[i].DY)
            g = getbearingbetween(tSHIP[i].OX, tSHIP[i].OY, tSHIP[i].X, tSHIP[i].Y)
            if getanglediff(j, g)<10 then
              tSHIP[i].TX = tSHIP[i].DX
              tSHIP[i].TY = tSHIP[i].DY
              tSHIP[i].Behaviour = "Inbound"
              tSHIP[i].Sail = "Full sail"
            else
              tSHIP[i].TX = tSHIP[i].OX + (math.cos(math.rad(bearingtoangle(boundangle(g+5))))*((tISLAND[tSHIP[i].Origin].Size/2)+225))
              tSHIP[i].TY = tSHIP[i].OY + (math.sin(math.rad(bearingtoangle(boundangle(g+5))))*((tISLAND[tSHIP[i].Origin].Size/2)+225))
            end
          end
        elseif tSHIP[i].Behaviour == "Patrol" then
          -- If I'm reasonably close to my target, set a new one, after ten tries head inbound to an island
          g = distancebetween(tSHIP[i].X, tSHIP[i].Y, tSHIP[i].TX, tSHIP[i].TY)
          if g<=50 then
            tSHIP[i].Patrol = tSHIP[i].Patrol + 1
            if tSHIP[i].Patrol <= 10 then
              tSHIP[i].TX, tSHIP[i].TY = tSHIP[i].TX+math.random(800)-math.random(800), tSHIP[i].TY+math.random(800)-math.random(800)
              while not getshiplocationfree(tSHIP[i].TX, tSHIP[i].TY) do
                tSHIP[i].TX, tSHIP[i].TY = tSHIP[i].TX+math.random(800)-math.random(800), tSHIP[i].TY+math.random(800)-math.random(800)
              end
            else
              j = getneartargetisland(tSHIP[i])
              tSHIP[i].Dest = j
              tSHIP[i].DX, tSHIP[i].DY = tISLAND[j].X, tISLAND[j].Y
              tSHIP[i].Patrol = 0
              tSHIP[i].TX, tSHIP[i].TY = tISLAND[j].X, tISLAND[j].Y
              tSHIP[i].Behaviour = "Inbound"
            end
          end
        elseif tSHIP[i].Behaviour == "Attack" then
          -- Get distance to target
          if tSHIP[i].Target == -1 then 
            tx, ty = pSHIP.X, pSHIP.Y
            g = distancebetween(tSHIP[i].X, tSHIP[i].Y, tx, ty)
          elseif tSHIP[i].Target >= 1 and tSHIP[i].Target<=gettablesize(tSHIP) then 
            tx, ty = tSHIP[tSHIP[i].Target].X, tSHIP[tSHIP[i].Target].Y
            g = distancebetween(tSHIP[i].X, tSHIP[i].Y, tx, ty)
          else  -- This shouldn't happen but just in case
            tSHIP[i].Behaviour = "Patrol"
            tSHIP[i].TX, tSHIP[i].TY = tSHIP[i].X, tSHIP[i].Y
            tSHIP[i].Target = 0
            g = -1
          end
          if g>1200 then
            -- If my target is more than 1.5screens away, just forget about them
            tSHIP[i].Behaviour = "Patrol"
            tSHIP[i].TX, tSHIP[i].TY = tSHIP[i].X, tSHIP[i].Y
            tSHIP[i].Target = 0
          elseif g>getcannonrange(tSHIP[i]) then
            -- If my target is outside of my shooting range just continue to head straight for them
            if tSHIP[i].Target==-1 then tSHIP[i].TX, tSHIP[i].TY = pSHIP.X, pSHIP.Y
            else tSHIP[i].TX, tSHIP[i].TY = tSHIP[tSHIP[i].Target].X, tSHIP[tSHIP[i].Target].Y end
          elseif g>0 then
            -- My target is within my range, am I in position for a shot?
            j = getanglediff(getbearingbetween(tSHIP[i].X, tSHIP[i].Y, tx, ty), tSHIP[i].Z)
            if j>85 and j<95 and tSHIP[i].Firing==0 and tSHIP[i].Reloading==0 then
              -- That's close enough, take a shot!
              j = getbearingbetween(tSHIP[i].X, tSHIP[i].Y, tx, ty)
              if getanglediff(boundangle(tSHIP[i].Z+90), j)<getanglediff(boundangle(tSHIP[i].Z-90), j) then tSHIP[i].Cannons = "Right"
              else tSHIP[i].Cannons = "Left" end
              tSHIP[i].Firing = tSETTINGS.FPS
              if getisonscreen(tSHIP[i]) then playsound(7) end
            else
              -- Get myself in position for a shot
              tSHIP[i].TX = tSHIP[i].X + (math.cos(math.rad(bearingtoangle(boundangle(j-90))))*100)
              tSHIP[i].TY = tSHIP[i].Y + (math.sin(math.rad(bearingtoangle(boundangle(j-90))))*100)
            end
          else end
        else end
        -- Check for collisions with islands & make decisions to move if they're close
        avoiding = false
        for j = 1, #tISLAND, 1 do
          g = distancebetween(tSHIP[i].X, tSHIP[i].Y, tISLAND[j].X, tISLAND[j].Y)
          -- Dock at target if close enough and inbound and it's my target destination, go half sail at a distance
          if g<=(tISLAND[j].Size/2)+100 and j == tSHIP[i].Dest and tSHIP[i].Behaviour=="Inbound" then
            tSHIP[i].Behaviour = "Docked"
            tSHIP[i].Docked, tSHIP[i].Origin = j, j
            tSHIP[i].OX, tSHIP[i].OY = tISLAND[j].X, tISLAND[j].Y
            tSHIP[i].Dest = 0
            tSHIP[i].DX, tSHIP[i].DY = 0, 0
            engine = 0
            tSHIP[i].Anchor = "Down"
            if getisonscreen(tSHIP[i]) then playsound(2, "Yes") end
            tSHIP[i].Resting = tSETTINGS.FPS*60*math.random(5)
          elseif g<=(tISLAND[j].Size/2)+150 and j==tSHIP[i].Dest and tSHIP[i].Behaviour=="Inbound" then
            engine = math.min(engine, 1)
          else end
          -- If close to an island that is not my target and I'm inbound, or I'm attacking, orbit it
          if g<=(tISLAND[j].Size/2)+150 and j~=tSHIP[i].Dest and tSHIP[i].Behaviour=="Inbound" then
            tSHIP[i].Behaviour = "Orbit"
            engine = math.min(engine, 1)
            tSHIP[i].Origin = j
            tSHIP[i].OX, tSHIP[i].OY = tISLAND[j].X, tISLAND[j].Y
          end  
          -- If close to an island and I'm patrolling or attacking, set target away from here and make a note I'm avoiding
          if g<=(tISLAND[j].Size/2)+150 and (tSHIP[i].Behaviour=="Patrol" or tSHIP[i].Behaviour=="Attack") then
            turn = getbearingbetween(tISLAND[j].X, tISLAND[j].Y, tSHIP[i].X, tSHIP[i].Y)
            tSHIP[i].TX = tSHIP[i].X + (math.cos(math.rad(bearingtoangle(turn)))*150)
            tSHIP[i].TY = tSHIP[i].Y + (math.sin(math.rad(bearingtoangle(turn)))*150)
            avoiding = true
          end
          -- If everything has gone wrong I crash into the island
          if g<=(tISLAND[j].Size/2)+100 then
            if checkislandcollision(tSHIP[i], tISLAND[j]) == true then collideisland(tSHIP[i]) end
          end
        end
        -- Check for collisions with other ships & make decisions
        for j = 1, #tSHIP, 1 do
          g = distancebetween(tSHIP[i].X, tSHIP[i].Y, tSHIP[j].X, tSHIP[j].Y)
          -- If I'm close enough to an enemy, attack them, unless I'm avoiding a crash
          if i~=j and g<=tSHIP[i]["Attack"..tSHIP[j].Type] and avoiding == false then
            tSHIP[i].Behaviour = "Attack"
            tSHIP[i].Target = j
            tSHIP[i].TX, tSHIP[i].TY = tSHIP[j].X, tSHIP[j].Y
            tSHIP[i].Anchor = "Up"
          end
          -- If I'm close to another ship that's not me... avoid it with a space 300 away 90degrees left!
          if i~=j and g<=(tSHIP[i].Height*1.3) then
            if tSHIP[i].Anchor=="Up" and avoiding==false then   -- Only dodge if my anchor is up
              g = getbearingbetween(tSHIP[i].X, tSHIP[i].Y, tSHIP[j].X, tSHIP[j].Y)
              tSHIP[i].TX = tSHIP[i].X + (math.cos(math.rad(bearingtoangle(boundangle(g-90))))*300)
              tSHIP[i].TY = tSHIP[i].Y + (math.sin(math.rad(bearingtoangle(boundangle(g-90))))*300)
              avoiding = true
            elseif tSHIP[i].Anchor=="Up" then engine=0
            else end
            -- If everything has gone wrong I crash into the ship
            if checkshipcollision(tSHIP[i], tSHIP[j]) == true then collideship(tSHIP[i]) end
          end
        end
        g = distancebetween(tSHIP[i].X, tSHIP[i].Y, pSHIP.X, pSHIP.Y)
        -- If I'm close enough to the player as an enemy, attack them as the priority (unless I'm avoiding a crash)!
        if g<=tSHIP[i].AttackP and avoiding == false then
          tSHIP[i].Behaviour = "Attack"
          tSHIP[i].Target = -1
          tSHIP[i].TX, tSHIP[i].TY = pSHIP.X, pSHIP.Y
          tSHIP[i].Anchor = "Up"
        end
        -- If I'm close to the player's ship, try to avoid it in the same way
        if g<=(tSHIP[i].Height)*1.3 then
          if tSHIP[i].Anchor=="Up" and avoiding==false then   -- Only dodge if my anchor is up
            g = getbearingbetween(tSHIP[i].X, tSHIP[i].Y, pSHIP.X, pSHIP.Y)
            tSHIP[i].TX = tSHIP[i].X + (math.cos(math.rad(bearingtoangle(boundangle(g-90))))*300)
            tSHIP[i].TY = tSHIP[i].Y + (math.sin(math.rad(bearingtoangle(boundangle(g-90))))*300)
          elseif tSHIP[i].Anchor=="Up" then engine=0
          else end
          -- If everything has gone wrong I crash into the ship
          if checkshipcollision(tSHIP[i], pSHIP) == true then collideship(tSHIP[i]) end
        end
        -- Now steer the ship towards its target
        j = getbearingbetween(tSHIP[i].X, tSHIP[i].Y, tSHIP[i].TX, tSHIP[i].TY)
        g = getanglediff(j, tSHIP[i].Z)
        if g<2 then -- Do nothing, that's close enough to the right direction
        else
          -- Minimise engine depending upon how great an angle it is to turn
          if g>=30 then engine = 0
          elseif g>=10 then engine = math.min(engine, 1)
          else end
        end
        -- Now set their sail get their turning ability based upon their engine (sail)
        if engine==0 then 
          tSHIP[i].Sail="Oars only"
          turn=getmaxturn(tSHIP[i])
        elseif engine==1 then 
          tSHIP[i].Sail="Half sail"
          turn=getmaxturn(tSHIP[i])/2
        else 
          tSHIP[i].Sail="Full sail"
          turn=getmaxturn(tSHIP[i])/4 
        end
        -- Time to steer the boat if we need to!
        if g>=2 then
          if getanglediff(tSHIP[i].Z+turn, j)<getanglediff(tSHIP[i].Z-turn, j) then
            tSHIP[i].Z = boundangle(tSHIP[i].Z + turn)
          else
            tSHIP[i].Z = boundangle(tSHIP[i].Z - turn)
          end
        end
      end 
      -- Dying ships remain for 2 seconds
      if tSHIP[i].Dying > 0 then
        tSHIP[i].Dying = tSHIP[i].Dying - 1
        if tSHIP[i].Dying == 0 then tSHIP[i].Delete = "Yes" end
      elseif gethealth(tSHIP[i])==0 then 
        tSHIP[i].Dying = tSETTINGS.FPS * 2
        -- Am I anyone's target? If I am, clear that target so they don't get confused
        for j = 1, #tSHIP, 1 do
          if i~=j and tSHIP[j].Target==i then
            tSHIP[j].Target = 0
            tSHIP[j].Behaviour="Patrol"
            tSHIP[j].TX, tSHIP[j].TY = tSHIP[j].X, tSHIP[j].Y
          end
        end
      else end  
      -- Remove ships that are a long, long way away (3 screenwidths)
      if distancebetween(tSHIP[i].X, tSHIP[i].Y, pSHIP.X, pSHIP.Y)>2400 then tSHIP[i].Delete = "Yes" end
      -- Remove ships due for deletion
      if tSHIP[i].Delete == "Yes" then 
        -- Am I anyone's target? If I am, clear that target so they don't get confused
        for j = 1, #tSHIP, 1 do
          if i~=j and tSHIP[j].Target==i then
            tSHIP[j].Target = 0
            tSHIP[j].Behaviour="Patrol"
            tSHIP[j].TX, tSHIP[j].TY = tSHIP[j].X, tSHIP[j].Y
          end
        end
        table.remove(tSHIP, i) 
      end
    end
  end
end

function updatespawnships()   -- Spawn new ships if space
  local g, near, near_ref = 0, math.huge, 0
  local i, j, tot, x, y, z
  -- Spawn new ships if there's space
  if gettablesize(tSHIP)<tSETTINGS.MaxShips and tSETTINGS.MaxShips-gettablesize(tSHIP)>=math.random(tSETTINGS.SpawnRate) then
    for i = 1, #tISLAND, 1 do
      j = distancebetween(tISLAND[i].X, tISLAND[i].Y, pSHIP.X, pSHIP.Y)
      if j<near then
        near_ref = i      -- this is the closest island to the player
        near = j          -- this is how close that island is
      end
    end
    for i = 1, #tGOVERNMENT, 1 do
      if tGOVERNMENT[i].Name == tISLAND[near_ref].Government then g = i end   -- this is the government type of the closest island
    end
    -- Decide which ship to spawn based off the government type of nearest island
    if g~=0 then
      tot, i, j = 0, 0, math.random(100)
      while tot < j do    -- Loop through the spawn percentages until the total is equal to or exceeds the random number
        i = i + 1
        tot = tot + tGOVERNMENT[g]["Spawn"..i]
      end
      z, j = math.random(360)-1, 550+math.random(250)   -- Random angle around player, at a set distance plus a random
      x, y = pSHIP.X+(math.cos(math.rad(bearingtoangle(z)))*j), pSHIP.Y+(math.sin(math.rad(bearingtoangle(z)))*j)
      while not getshiplocationfree(x, y) do
        z, j = math.random(360)-1, 550+math.random(250)   -- Random angle around player, at a set distance plus a random
        x, y = pSHIP.X+(math.cos(math.rad(bearingtoangle(z)))*j), pSHIP.Y+(math.sin(math.rad(bearingtoangle(z)))*j)
      end
      spawnship(x, y, i, math.random(100))
      tSHIP[#tSHIP].Origin = near_ref                                                        -- Origin island
      tSHIP[#tSHIP].OX, tSHIP[#tSHIP].OY = tISLAND[near_ref].X, tISLAND[near_ref].Y          -- Origin coords
      near_ref = getneartargetisland(tSHIP[#tSHIP])                                          -- Find a suitable nearby island to aim for
      tSHIP[#tSHIP].Dest = near_ref                                                          -- Target/Destination island
      tSHIP[#tSHIP].DX, tSHIP[#tSHIP].DY = tISLAND[near_ref].X, tISLAND[near_ref].Y           -- Destination coords
    end
  end
end

function updatebullets()   -- Updates to cannon balls etc.
  local collided
  local i, j
  if next(tBULLET)~=nil then
    for i = #tBULLET, 1, -1 do
      tBULLET[i].X = tBULLET[i].X + (math.cos(math.rad(bearingtoangle(tBULLET[i].Z)))*tBULLET[i].Speed)
      tBULLET[i].Y = tBULLET[i].Y + (math.sin(math.rad(bearingtoangle(tBULLET[i].Z)))*tBULLET[i].Speed)
      tBULLET[i].Count = math.max(tBULLET[i].Count-tBULLET[i].Speed, 0)
      if tBULLET[i].Count == 0 then 
        if getisonscreen(tBULLET[i]) then playsound(16, "Yes") end
        table.remove(tBULLET, i) 
      else
        collided = false    -- Deal with removing the bullet later if it has collided, can hit two things at once
        -- Check for collisions with islands
        for j = 1, #tISLAND, 1 do
          if distancebetween(tBULLET[i].X, tBULLET[i].Y, tISLAND[j].X, tISLAND[j].Y)<=math.sqrt(2*((tISLAND[j].Size/2)^2)) then 
            if checkislandcollision(tBULLET[i], tISLAND[j]) == true then
              spawnexplosion(tBULLET[i].X, tBULLET[i].Y)
              if getisonscreen(tBULLET[i]) then playsound(13, "Yes") end
              collided = true
            end
          end
        end
        -- Check for collisions with ships
        if next(tSHIP)~=nil then
          for j = 1, #tSHIP, 1 do
            if distancebetween(tBULLET[i].X, tBULLET[i].Y, tSHIP[j].X, tSHIP[j].Y)<=(tBULLET[i].Height/2)+(tSHIP[j].Height/2) then
              if checkbulletcollision(tBULLET[i], tSHIP[j]) == true then
                spawnexplosion(tBULLET[i].X, tBULLET[i].Y)
                if getisonscreen(tBULLET[i]) then playsound(13, "Yes") end
                damageship(tSHIP[j], tBULLET[i].Damage)
                if tBULLET[i].Owner=="Player" then 
                  if tSHIP[j].AttackP == 0 then addfloatytext("Captain "..tSHIP[j].PlayerName.." is outraged and will return fire!") end
                  tSHIP[j].AttackP = 800
                else tSHIP[j]["Attack"..string.sub(tBULLET[i].Owner,5,5)] = 800 end
                collided = true
              end
            end
          end
        end
        -- Check for collision with player ship
        if distancebetween(tBULLET[i].X, tBULLET[i].Y, pSHIP.X, pSHIP.Y)<=(tBULLET[i].Height/2)+(pSHIP.Height/2) then
          if checkbulletcollision(tBULLET[i], pSHIP) == true then
            spawnexplosion(tBULLET[i].X, tBULLET[i].Y)
            if getisonscreen(tBULLET[i]) then playsound(13, "Yes") end
            damageship(pSHIP, tBULLET[i].Damage)
            collided = true
          end
        end
        if collided == true then
          table.remove(tBULLET, i)
        end
      end
    end
  end
end

function updateeffects()   -- Updates to explosions etc.
  local i
  if next(tEXPLODE)~=nil then
    for i = #tEXPLODE, 1, -1 do
      tEXPLODE[i].Count = math.max(tEXPLODE[i].Count - 1, 0)
      if tEXPLODE[i].Count == 0 then
        if tEXPLODE[i].Version < #iEXPLODE then
          tEXPLODE[i].Version = tEXPLODE[i].Version + 1
          tEXPLODE[i].Count = tEXPLODE[i].Refresh
        else table.remove(tEXPLODE, i) end
      end
    end
  end
end

function checkislandcollision(ship, island)   -- Is the ship colliding with the island?
  local crashed = false
  local r, g, b, a, i, p, q
  local test_x = {ship.X+(math.cos(math.rad(bearingtoangle(ship.Z)))*(ship.Height/2)), 
                  ship.X, 
                  ship.X-(math.cos(math.rad(bearingtoangle(ship.Z)))*(ship.Height/2))}
  local test_y = {ship.Y+(math.sin(math.rad(bearingtoangle(ship.Z)))*(ship.Height/2)), 
                  ship.Y, 
                  ship.Y-(math.sin(math.rad(bearingtoangle(ship.Z)))*(ship.Height/2))}
  for i = 1, 3, 1 do
    if test_x[i] > island.X-(island.Size/2) and test_x[i] < island.X+(island.Size/2) and test_y[i] > island.Y-(island.Size/2) and test_y[i] < island.Y+(island.Size/2) then
      p = test_x[i]-(island.X-(island.Size/2))     -- How far across the island am I in game coordinates?
      q = test_y[i]-(island.Y-(island.Size/2))     -- How far up the island am I in game coordinates?
      p, q = p*island.Image:getWidth()/island.Size, q*island.Image:getHeight()/island.Size        -- Scale these coordinates to the actual image size
      p, q = math.floor(p+0.5), math.floor((island.Image:getHeight()-q)+0.5)   -- Round the image coordinates, flip the y-coordinate
      if p>=0 and p<island.Image:getWidth() and q>=0 and q<island.Image:getHeight() then
        r, g, b, a = island.Data:getPixel(p, q)
        if a~=0 then crashed = true end
      end
    end
  end
  return crashed
end

function checkshipcollision(ship1, ship2)     -- Is ship1 colliding into ship2?
  local x1, y1 = ship1.X+(math.cos(math.rad(bearingtoangle(ship1.Z)))*(ship1.Height/2)), ship1.Y+(math.sin(math.rad(bearingtoangle(ship1.Z)))*(ship1.Height/2))
  local x2, y2 = ship1.X-(math.cos(math.rad(bearingtoangle(ship1.Z)))*(ship1.Height/2)), ship1.Y-(math.sin(math.rad(bearingtoangle(ship1.Z)))*(ship1.Height/2))
  local x3, y3 = ship2.X+(math.cos(math.rad(bearingtoangle(ship2.Z)))*(ship2.Height/2)), ship2.Y+(math.sin(math.rad(bearingtoangle(ship2.Z)))*(ship2.Height/2))
  local x4, y4 = ship2.X-(math.cos(math.rad(bearingtoangle(ship2.Z)))*(ship2.Height/2)), ship2.Y-(math.sin(math.rad(bearingtoangle(ship2.Z)))*(ship2.Height/2))
  local uA = ((x4-x3)*(y1-y3) - (y4-y3)*(x1-x3)) / ((y4-y3)*(x2-x1) - (x4-x3)*(y2-y1))
  local uB = ((x2-x1)*(y1-y3) - (y2-y1)*(x1-x3)) / ((y4-y3)*(x2-x1) - (x4-x3)*(y2-y1))
  if uA>=0 and uA<=1 and uB>=0 and uB<=1 then return true end
  if distancebetween(ship1.X, ship1.Y, ship2.X, ship2.Y)<=(ship1.Width/2)+(ship2.Width/2) then return true end
  return false
end

function checkbulletcollision(bullet, ship)
  local x1, y1 = ship.X+(math.cos(math.rad(bearingtoangle(ship.Z)))*(ship.Height/2)), ship.Y+(math.sin(math.rad(bearingtoangle(ship.Z)))*(ship.Height/2))
  local x2, y2 = ship.X-(math.cos(math.rad(bearingtoangle(ship.Z)))*(ship.Height/2)), ship.Y-(math.sin(math.rad(bearingtoangle(ship.Z)))*(ship.Height/2))
  local len, dot, cx, cy, d1, d2
  if distancebetween(x1, y1, bullet.X, bullet.Y)<=bullet.Width/2 then return true end
  if distancebetween(x2, y2, bullet.X, bullet.Y)<=bullet.Width/2 then return true end
  len = distancebetween(x1, y1, x2, y2)
  dot = (((bullet.X-x1)*(x2-x1))+((bullet.Y-y1)*(y2-y1)))/(len^2)
  cx, cy = x1+(dot*(x2-x1)), y1+(dot*(y2-y1))
  d1, d2 = distancebetween(cx, cy, x1, y1), distancebetween(cx, cy, x2, y2)
  if d1+d2>=len-0.1 and d1+d2<=len+0.1 then
    if distancebetween(cx, cy, bullet.X, bullet.Y)<=bullet.Width/2 then return true end
  end
  return false
end

function love.draw()  -- Draw everything in, based on gamemode
  love.graphics.scale(gRATIOX, gRATIOY)
  if _G["draw"..gGAMEMODE]~=nil then _G["draw"..gGAMEMODE]() end
  if gGAMEMODE ~= "help" and gGAMEMODE ~= "gameover" then drawfloatytext() end
  if gFADE > 0 then 
    love.graphics.setColor(0, 0, 0, gFADE)
    love.graphics.rectangle("fill", 0, 0, 800, 600)
  else end
end

function drawnewgame()
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.draw(iBACKGROUND[6])
  drawgreybox()
  drawshadowtext("Creating World...", 0, 400, 800, "center", fSUBHEADING, 255, 255, 255, 255)
  love.graphics.setColor(255, 215, 0, 255)
  love.graphics.rectangle("line", 198, 498, 404, 64)
  love.graphics.rectangle("fill", 200, 500, 400*gSELECT/tSETTINGS.Islands, 60)
end

function drawshadowtext(text, x, y, limit, align, font, r, g, b, a)
  local o
  if font==fTITLE then o = 3
  elseif font==fHEADING then o = 2
  else o = 1 end
  love.graphics.setFont(font)
  love.graphics.setColor(0, 0, 0, a)
  love.graphics.printf(text, x+o, y+o, limit, align)
  love.graphics.setColor(r, g, b, a)
  love.graphics.printf(text, x, y, limit, align)
end

function drawmenu()  -- Draw the main menu
  local mx, my = love.mouse.getX()/gRATIOX, love.mouse.getY()/gRATIOY
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.draw(iBACKGROUND[1], 0, 0)
  love.graphics.setFont(fTITLE)
  love.graphics.setColor(255, 255, 255, 100)
  love.graphics.printf("Sea Traders", 3, 63, 800, "center")
  love.graphics.setColor(0, 0, 0, 255)
  love.graphics.printf("Sea Traders", 0, 60, 800, "center")
  love.graphics.setFont(fHEADING)
  love.graphics.setColor(0, 0, 0, 100)
  love.graphics.printf("New Game", 43, 303, 226, "left")
  love.graphics.printf("Quit", 457, 303, 226, "right")
  love.graphics.setColor(255, 255, 255, 255)
  if tSETTINGS.Music=="Yes" then love.graphics.draw(iICON[1], 0, 550)
  else love.graphics.draw(iICON[2], 0, 550) end
  if tSETTINGS.Sound=="Yes" then love.graphics.draw(iICON[16], 750, 550)
  else love.graphics.draw(iICON[17], 750, 550) end
  if mx>40 and mx<252 and my>300 and my<364 then
    love.graphics.printf("New Game", 40, 300, 226, "left")
  elseif mx>580 and mx<680 and my>300 and my<364 then
    love.graphics.printf("Quit", 454, 300, 226, "right")
  else end
  if love.filesystem.getInfo("save_player.txt", "file") then
    love.graphics.setFont(fTITLE)
    love.graphics.setColor(0, 0, 0, 100)
    love.graphics.printf("Continue", 269, 403, 266, "center")
    if mx>266 and mx<532 and my>400 and my<500 then
      love.graphics.setColor(255, 255, 255, 255)
      love.graphics.printf("Continue", 266, 400, 266, "center")
    end
  end
end

function drawchoosenames()
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.draw(iBACKGROUND[2], 0, 0)
  drawbox(400, 300, 700, 350)
  drawbox(400, 540, 400, 100)
  love.graphics.draw(iICON[13], 0, 0)
  drawshadowtext("Choose Names", 0, 10, 800, "center", fTITLE, 255, 215, 0, 255)
  drawshadowtext("RANDOM", 650, 270, 75, "left", fDEFAULT, 255, 255, 255, 255)
  drawshadowtext("RANDOM", 650, 410, 75, "left", fDEFAULT, 255, 255, 255, 255)
  drawshadowtext("Captain:", 0, 200, 250, "right", fHEADING, 255, 215, 0, 255)
  drawshadowtext("Boat:", 0, 340, 250, "right", fHEADING, 255, 215, 0, 255)
  drawshadowtext("Set Sail", 200, 505, 400, "center", fHEADING, 255, 255, 255, 255)
  if gTYPING==true and gSELECT==1 then drawshadowtext(gTYPEDTEXT, 275, 200, 525, "left", fHEADING, 255, 255, 255, 255)
  else drawshadowtext(pSHIP.PlayerName, 275, 200, 525, "left", fHEADING, 255, 215, 0, 255) end
  if gTYPING==true and gSELECT==2 then drawshadowtext(gTYPEDTEXT, 275, 340, 525, "left", fHEADING, 255, 255, 255, 255)
  else drawshadowtext(pSHIP.Name, 275, 340, 525, "left", fHEADING, 255, 215, 0, 255) end
end

function drawhelp()
  drawgame()
  drawgreybox()
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.draw(iMAP, 80, 60)
  drawshadowtext("CONTROLS", 130, 105, 260, "left", fDEFAULT, 255, 215, 0, 255)
  drawshadowtext("Left/Right arrow\nUp/Down arrow\nA\nC\nG\nI\nM\nP\nW\nX/Z\nEscape", 130, 130, 100, "right", fDEFAULT, 255, 215, 0, 255)
  drawshadowtext("Steer ship\nRaise/Lower sail\nRaise/Lower anchor\nToggle compass\nGo ashore island\nToggle ship information\nToggle map screen\nToggle personnel screen\nToggle wind report\nFire cannon right/left\nMain menu", 240, 130, 150, "left", fDEFAULT, 255, 255, 255, 255)
  drawshadowtext("Press any key to continue", 0, 540, 800, "center", fSUBHEADING, 255, 255, 255, 255)
  drawshadowtext("OBJECT OF THE GAME", 410, 105, 260, "left", fDEFAULT, 255, 215, 0, 255)
  drawshadowtext("You begin as the Captain of a small vessel. Your objective is to adventure your way to fame and fortune by exploring, trading and succeeding in battle.", 410, 130, 260, "left", fDEFAULT, 255, 255, 255, 255)
  drawshadowtext("TIPS FOR BEGINNERS", 410, 250, 260, "left", fDEFAULT, 255, 215, 0, 255)
  drawshadowtext("Start by going ashore the island. You should hire some crew members then explore to discover advantageous trade routes. You may wish to register as a certified cartographer of maps. Over time you will also need to pay close attention to the health and morale of your crew. They need occasional stops at the inn to rest and recover. If the overall morale drops too low, you may face a mutiny!", 410, 275, 260, "left", fDEFAULT, 255, 255, 255, 255)
end

function drawsail()
  drawgame()
  drawoverlay()
end

function drawsailmap()
  drawmap()
end

function drawsailcrew()
  drawgame()
  drawgreybox()
  drawcrew()
end

function drawgame()
  local i
  drawbackground()
  drawland()
  drawships()
  drawplayer()
  drawbullets()
  draweffects()
  drawlighting()
end

function drawbackground()
  local w, h = iOCEAN[gBG_Z]:getWidth(), iOCEAN[gBG_Z]:getHeight()
  local a, b = math.ceil(800/w)+1, math.ceil(600/h)+1
  local i, j
  love.graphics.setColor(255, 255, 255, 255)
  for i = -1, a, 1 do
    for j = -1, b, 1 do
      love.graphics.draw(iOCEAN[gBG_Z], gBG_X+(i*w), gBG_Y+(j*h))
    end
  end
end

function tilebackground()
  local w, h = iOCEAN[gBG_Z]:getWidth(), iOCEAN[gBG_Z]:getHeight()
  if gBG_X+w<0 then gBG_X=0 end
  if gBG_X>w then gBG_X=0 end
  if gBG_Y+h<0 then gBG_Y=0 end
  if gBG_Y>h then gBG_Y=0 end
end

function drawland()
  local i, x, y, w, h
  local r = getscoutrange()
  for i = 1, #tISLAND, 1 do
    if distancebetween(pSHIP.X, pSHIP.Y, tISLAND[i].X, tISLAND[i].Y)<=650+(tISLAND[i].Size/2) then
      if distancebetween(pSHIP.X, pSHIP.Y, tISLAND[i].X, tISLAND[i].Y)<=(tISLAND[i].Size/2)+r then 
        if tISLAND[i].Discovered ~= "Yes" then 
          addfloatytext("Land ho! You have discovered "..tISLAND[i].Name..".") 
          if pSHIP.Cartographer ~= -1 then pSHIP.Cartographer = pSHIP.Cartographer + 1 end
        end
        tISLAND[i].Discovered = "Yes" 
      end
      x, y = worldtoscreen(tISLAND[i].X, tISLAND[i].Y)
      w, h = tISLAND[i].Image:getWidth(), tISLAND[i].Image:getHeight()
      love.graphics.setColor(200, 200, 200, 255)
      drawdottedcircle(x, y, (tISLAND[i].Size/2)+150)
      love.graphics.setColor(255, 255, 255, 255)
      love.graphics.draw(tISLAND[i].Image, x, y, 0, tISLAND[i].Size/w, tISLAND[i].Size/h, w/2, h/2)
      if tSETTINGS.Names=="Yes" then
        if tISLAND[i].Size<=100 then drawshadowtext(tISLAND[i].Name, x-400, y-12, 800, "center", fDEFAULT, 255, 255, 255, 255)
        elseif tISLAND[i].Size<=200 then 
          drawshadowtext(tISLAND[i].Name, x-400, y-116, 800, "center", fSUBHEADING, 255, 255, 255, 255)
          drawshadowtext(tISLAND[i].Name, x-400, y+100, 800, "center", fSUBHEADING, 255, 255, 255, 255)
        else
          drawshadowtext(tISLAND[i].Name, x-400, y-232, 800, "center", fHEADING, 255, 255, 255, 255) 
          drawshadowtext(tISLAND[i].Name, x-600, y-32, 600, "center", fHEADING, 255, 255, 255, 255)
          drawshadowtext(tISLAND[i].Name, x, y-32, 600, "center", fHEADING, 255, 255, 255, 255) 
          drawshadowtext(tISLAND[i].Name, x-400, y+200, 800, "center", fHEADING, 255, 255, 255, 255) 
        end
      end
    end
  end
end

function drawplayer()
  local h, v = gethealthpercent(pSHIP), 1
  if h==0 then v = 5
  elseif h<=10 then v = 4
  elseif h<=40 then v = 3
  elseif h<=70 then v = 2
  else v = 1 end
  if getisnight() and pSHIP.Upgrades[1]=="Yes" then
    love.graphics.setColor(255, 255, 255, 125)
    love.graphics.draw(iLANTERN, 350, 250)
  end
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.draw(pSHIP.Image[v], 400, 300, math.rad(pSHIP.Z), 1, 1, pSHIP.Image[v]:getWidth()/2, pSHIP.Image[v]:getHeight()/2)
  if tSETTINGS.Names=="Yes" then
    drawshadowtext("The "..pSHIP.Name, 300, 340, 200, "center", fSMALL, 255, 215, 0, 255)
    drawshadowtext("Captain "..pSHIP.PlayerName, 300, 360, 200, "center", fSMALL, 255, 255, 255, 255)
  end
end

function drawships()
  local i, h, v, x, y
  if next(tSHIP)~=nil then
    for i = 1, #tSHIP, 1 do
      h, v = gethealthpercent(tSHIP[i]), 1
      if h==0 then v = 5
      elseif h<=10 then v = 4
      elseif h<=40 then v = 3
      elseif h<=70 then v = 2
      else v = 1 end
      x, y = worldtoscreen(tSHIP[i].X, tSHIP[i].Y)
      if getisnight() and tSHIP[i].Upgrades[1]=="Yes" then
        love.graphics.setColor(255, 255, 255, 125)
        love.graphics.draw(iLANTERN, x, y, 0, 1, 1, iLANTERN:getWidth()/2, iLANTERN:getHeight()/2)
      end
      love.graphics.setColor(255, 255, 255, 255)
      love.graphics.draw(tSHIP[i].Image[v], x, y, math.rad(tSHIP[i].Z), 1, 1, tSHIP[i].Image[v]:getWidth()/2, tSHIP[i].Image[v]:getHeight()/2)
      if tSETTINGS.Names=="Yes" then
        drawshadowtext("The "..tSHIP[i].Name, x-100, y+40, 200, "center", fSMALL, 255, 215, 0, 255)
        drawshadowtext("Captain "..tSHIP[i].PlayerName, x-100, y+60, 200, "center", fSMALL, 255, 255, 255, 255)
      end
    end
  end
end

function drawbullets()
  local i, x, y, w, h
  if next(tBULLET)~=nil then
    love.graphics.setColor(255, 255, 255, 255)
    w, h = iICON[7]:getWidth(), iICON[7]:getHeight()
    for i = 1, #tBULLET, 1 do
      if distancebetween(pSHIP.X, pSHIP.Y, tBULLET[i].X, tBULLET[i].Y)<=550 then
        x, y = worldtoscreen(tBULLET[i].X, tBULLET[i].Y)
        love.graphics.draw(iICON[7], x, y, 0, 12/w, 12/h, w/2, h/2)
      end
    end
  end
end

function draweffects()
  local i, x, y, w, h
  if next(tEXPLODE)~=nil then
    love.graphics.setColor(255, 255, 255, 255)
    for i = 1, #tEXPLODE, 1 do
      if distancebetween(pSHIP.X, pSHIP.Y, tEXPLODE[i].X, tEXPLODE[i].Y)<=550 then
        x, y = worldtoscreen(tEXPLODE[i].X, tEXPLODE[i].Y)
        w, h = iEXPLODE[tEXPLODE[i].Version]:getWidth(), iEXPLODE[tEXPLODE[i].Version]:getHeight()
        love.graphics.draw(iEXPLODE[tEXPLODE[i].Version], x, y, 0, 1, 1, w/2, h/2)
      end
    end
  end
end

function drawlighting()
  local r, g, b, a = getlighting()
  love.graphics.setColor(r, g, b, a)
  love.graphics.rectangle("fill", 0, 0, 800, 600)
end

function drawoverlay()
  if tSETTINGS.Compass=="Yes" then drawcompass(100, 490) end
  drawclock(60, 40)
  if tSETTINGS.Wind=="Yes" then drawwind(60, 110) end
  if tSETTINGS.Info=="Yes" then drawstats(740, 80) end
  drawicons()
end

function drawcompass(x,y)
  local cx, cy, cn = {x-15, x+65, x-12, x-95}, {y-105, y-14, y+65, y-14}, {"N", "E", "S", "W"}
  local i, course
  love.graphics.setColor(0, 0, 0, 125)
  love.graphics.circle("fill", x, y, 90)
  for i = 1, 4, 1 do
    drawshadowtext(cn[i], cx[i], cy[i], 100, "left", fSUBHEADING, 255, 255, 255, 255)
  end
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.draw(iICON[3], x, y, 0, 160/iICON[3]:getWidth(), 160/iICON[3]:getHeight(), iICON[3]:getWidth()/2, iICON[3]:getHeight()/2)
  if pSHIP.TX~=nil and pSHIP.TY~=nil and (pSHIP.X~=pSHIP.TX or pSHIP.Y~=pSHIP.TY) then
    course = getbearingbetween(pSHIP.X, pSHIP.Y, pSHIP.TX, pSHIP.TY)
    love.graphics.setColor(200, 0, 0, 255)
    love.graphics.line(x, y, x+(math.cos(math.rad(bearingtoangle(course)))*90), y-(math.sin(math.rad(bearingtoangle(course)))*90))
  end  
  love.graphics.setColor(0, 200, 0, 255)
  love.graphics.line(x, y, x+(math.cos(math.rad(bearingtoangle(pSHIP.Z)))*80), y-(math.sin(math.rad(bearingtoangle(pSHIP.Z)))*80))
end

function drawclock(x,y)
  drawbox(x, y, 100, 60)
  drawshadowtext("Day "..tCLOCK.Day, x-40, y-25, 80, "center", fDEFAULT, 255, 255, 255, 255)
  drawshadowtext("Hour "..tCLOCK.Hour, x-40, y+5, 80, "center", fDEFAULT, 255, 255, 255, 255)
end

function drawwind(x,y)
  drawbox(x, y, 100, 60)
  drawshadowtext("Wind "..math.floor(tWIND.Speed+0.5).." knots", x-50, y-25, 100, "center", fDEFAULT, 255, 255, 255, 255)
  drawshadowtext(bearingtotext(tWIND.Direction), x-50, y+5, 100, "center", fDEFAULT, 255, 255, 255, 255)
end

function drawstats(x,y)
  local sn = {math.floor(pSHIP.Gold), gettablesize(pSHIP.Cargo).."/"..getcapacity(pSHIP), math.ceil(pSHIP.Food), pSHIP.Ammo, gethealthpercent(pSHIP).."%"} 
  local i
  drawbox(x, y, 100, 140)
  love.graphics.setFont(fDEFAULT)
  for i = 1, #sn, 1 do
    love.graphics.setColor(255, 255, 255, 255)
    love.graphics.draw(iICON[i+3], 715, (i*25)+5, 0, 24/iICON[i+3]:getWidth(), 24/iICON[i+3]:getHeight(), iICON[i+3]:getWidth()/2, iICON[i+3]:getHeight()/2)
    love.graphics.setColor(0, 0, 0, 255)
    love.graphics.printf(sn[i], 735, (i*25)-3, 55, "left")
  end
end

function drawicons()
  local i
  love.graphics.setColor(255, 255, 255, 255)
  if pSHIP.Firing > 0 then
    love.graphics.arc("fill", 758, 494, 24, math.rad(270-(pSHIP.Firing*360/tSETTINGS.FPS)), math.rad(-90))
  elseif pSHIP.Reloading > 0 then
    love.graphics.arc("fill", 758, 494, 24, math.rad(-90), math.rad((pSHIP.Reloading*360/(math.max(pSHIP.Reload-getreloadspeed(pSHIP),50)))-90))
  else end
  if pSHIP.Anchor=="Down" then 
    love.graphics.draw(iICON[9], 726, 526)
    i = getislanddocked(pSHIP.X, pSHIP.Y)
    if i~=0 then
      drawbox(400, 45, 200, 70)
      love.graphics.setFont(fDEFAULT)
      love.graphics.setColor(0, 0, 0, 255)
      if getplayerwelcome(i) then
        love.graphics.printf("Welcome to "..tISLAND[i].Name..".", 300, 18, 200, "center")
        love.graphics.printf("Press 'G' to go ashore.", 300, 36, 200, "center")
        love.graphics.printf("Press 'A' to go haul anchor.", 300, 54, 200, "center")
      else love.graphics.printf("You are not welcome here!", 300, 18, 200, "center") end
    end
  elseif pSHIP.Sail=="Oars only" then love.graphics.draw(iICON[10], 726, 526)
  elseif pSHIP.Sail=="Half sail" then love.graphics.draw(iICON[11], 726, 526)
  elseif pSHIP.Sail=="Full sail" then love.graphics.draw(iICON[12], 726, 526)
  else end
end

function drawgreybox()
  love.graphics.setColor(0, 0, 0, 200)
  love.graphics.rectangle("fill", 0, 0, 800, 600)
end

function drawmap()
  local mx, my = love.mouse.getX()/gRATIOX, love.mouse.getY()/gRATIOY
  local i, w, h, x, y
  local dx, dy, steps
  love.graphics.setColor(200, 200, 200, 255)
  love.graphics.draw(iMAP, 80, 60)
  if pSHIP.TX~=nil and pSHIP.TY~=nil then
    steps = math.floor((distancebetween(pSHIP.X, pSHIP.Y, pSHIP.TX, pSHIP.TY)/50)+0.5)
    if steps >= 1 then
      dx, dy = (pSHIP.TX-pSHIP.X)/steps, (pSHIP.TY-pSHIP.Y)/steps
      x, y = pSHIP.X, pSHIP.Y
      love.graphics.setColor(0, 0, 0, 255)
      for i = 1, steps, 1 do
        x,y = x+dx, y+dy
        if (x-gMAPX)*tSETTINGS.MapZoom>-270 and (x-gMAPX)*tSETTINGS.MapZoom<270 and (y-gMAPY)*tSETTINGS.MapZoom>-190 and (y-gMAPY)*tSETTINGS.MapZoom<190 then love.graphics.circle("fill", 400+((x-gMAPX)*tSETTINGS.MapZoom), 300-((y-gMAPY)*tSETTINGS.MapZoom), 1) end
      end
    end
  end
  for i = 1, #tISLAND, 1 do
    if tISLAND[i].Discovered == "Yes" then
      x, y = (tISLAND[i].X-gMAPX)*tSETTINGS.MapZoom, (tISLAND[i].Y-gMAPY)*tSETTINGS.MapZoom
      if (x-(tISLAND[i].Size*tSETTINGS.MapZoom/2)<400 or x+(tISLAND[i].Size*tSETTINGS.MapZoom/2)>-400) and (y-(tISLAND[i].Size*tSETTINGS.MapZoom/2)<300 or y+(tISLAND[i].Size*tSETTINGS.MapZoom/2)>-300) then
        love.graphics.setColor(200, 200, 200, 255)
        w, h = tISLAND[i].Image:getWidth(), tISLAND[i].Image:getHeight()
        love.graphics.draw(tISLAND[i].Image, 400+x, 300-y, 0, tSETTINGS.MapZoom*tISLAND[i].Size/w, tSETTINGS.MapZoom*tISLAND[i].Size/h, w/2, h/2)
        if tSETTINGS.MapZoom >= 0.15 then drawshadowtext(tISLAND[i].Name, x, 288-y, 800, "center", fDEFAULT, 255, 255, 255, 255)
        elseif tSETTINGS.MapZoom >= 0.1 then drawshadowtext(tISLAND[i].Name, x, 290-y, 800, "center", fSMALL, 255, 255, 255, 255)
        else drawshadowtext(tISLAND[i].Name, x, 292-y, 800, "center", fTINY, 255, 255, 255, 255) end
      end
    end
  end
  if pSHIP.TX~=nil and pSHIP.TY~=nil then
    if (pSHIP.TX-gMAPX)*tSETTINGS.MapZoom>-400 and (pSHIP.TX-gMAPX)*tSETTINGS.MapZoom<400 and (pSHIP.TY-gMAPY)*tSETTINGS.MapZoom>-300 and (pSHIP.TY-gMAPY)*tSETTINGS.MapZoom<300 then
      drawshadowtext("X", (pSHIP.TX-gMAPX)*tSETTINGS.MapZoom, 284-((pSHIP.TY-gMAPY)*tSETTINGS.MapZoom), 800, "center", fSUBHEADING, 255, 255, 255, 255)
    end
  end
  if (pSHIP.X-gMAPX)*tSETTINGS.MapZoom>-400 and (pSHIP.X-gMAPX)*tSETTINGS.MapZoom<400 and (pSHIP.Y-gMAPY)*tSETTINGS.MapZoom>-300 and (pSHIP.Y-gMAPY)*tSETTINGS.MapZoom<300 then
    love.graphics.setColor(200, 200, 200, 255)
    love.graphics.draw(pSHIP.Image[1], 400+((pSHIP.X-gMAPX)*tSETTINGS.MapZoom), 300-((pSHIP.Y-gMAPY)*tSETTINGS.MapZoom), math.rad(pSHIP.Z), tSETTINGS.MapZoom, tSETTINGS.MapZoom, pSHIP.Image[1]:getWidth()/2, pSHIP.Image[1]:getHeight()/2)
  end
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.draw(iFRAME, 0, 0)
  if mx>660 and mx<690 and my>460 and my<490 then love.graphics.setColor(255, 255, 255, 255)
  else love.graphics.setColor(255, 255, 255, 150) end
  love.graphics.draw(iICON[14], 675, 475, 0, 0.5, 0.5, iICON[14]:getWidth()/2, iICON[14]:getHeight()/2)
  if mx>660 and mx<690 and my>490 and my<520 then love.graphics.setColor(255, 255, 255, 255)
  else love.graphics.setColor(255, 255, 255, 150) end
  love.graphics.draw(iICON[15], 675, 505, 0, 0.5, 0.5, iICON[15]:getWidth()/2, iICON[15]:getHeight()/2)
  if pSHIP.TX~=nil and pSHIP.TY~=nil then drawshadowtext("Press spacebar to clear the current destination.", 0, 540, 800, "center", fDEFAULT, 255, 255, 255, 255)
  elseif gethasrole("Navigator") then drawshadowtext("Drag the map to move it. Ctrl-click to plot a course.", 0, 540, 800, "center", fDEFAULT, 255, 255, 255, 255)
  else drawshadowtext("Drag the map to move it.", 0, 540, 800, "center", fDEFAULT, 255, 255, 255, 255) end
  if gethasrole("Cartographer") then
    for i = 1, #tISLAND, 1 do
      if tISLAND[i].Discovered == "Yes" then
        x, y = (tISLAND[i].X-gMAPX)*tSETTINGS.MapZoom, (tISLAND[i].Y-gMAPY)*tSETTINGS.MapZoom
        if mx>400+x-(tISLAND[i].Size*tSETTINGS.MapZoom/2) and mx<400+x+(tISLAND[i].Size*tSETTINGS.MapZoom/2) and my>300-y-(tISLAND[i].Size*tSETTINGS.MapZoom/2) and my<300-y+(tISLAND[i].Size*tSETTINGS.MapZoom/2) then
          love.graphics.setColor(0, 0, 0, 180)
          love.graphics.rectangle("fill", mx-80, my+25, 160, 60)
          drawshadowtext(tISLAND[i].Name, mx-75, my+30, 150, "left", fDEFAULT, 255, 255, 255, 255)
          drawshadowtext(tISLAND[i].Government, mx-75, my+60, 150, "left", fDEFAULT, 255, 215, 0, 255)
        end
      end
    end
  end
end

function drawcrew()
  local mx, my = love.mouse.getX()/gRATIOX, love.mouse.getY()/gRATIOY
  local health, morale, wages = 0, 0, 0
  local i
  drawshadowtext("Personnel", 80, 75, 640, "center", fHEADING, 255, 215, 0, 255)
  drawshadowtext("Morale", 420, 145, 50, "center", fDEFAULT, 255, 215, 0, 255)
  drawshadowtext("Health", 470, 145, 50, "center", fDEFAULT, 255, 215, 0, 255)
  drawshadowtext("Wage", 530, 145, 100, "left", fDEFAULT, 255, 255, 255, 255)
  love.graphics.setFont(fDEFAULT)
  for i = 1, #pSHIP.Crew, 1 do
    health, morale, wages = health+pSHIP.Crew[i].Health,  morale+pSHIP.Crew[i].Morale, wages+pSHIP.Crew[i].Pay
    drawshadowtext(pSHIP.Crew[i].Role, 100, (35*i)+140, 100, "right", fDEFAULT, 255, 215, 0, 255)
    drawshadowtext(pSHIP.Crew[i].Name, 210, (35*i)+140, 200, "left", fDEFAULT, 255, 255, 255, 255)
    drawshadowtext(pSHIP.Crew[i].Morale.."%", 420, (35*i)+140, 50, "center", fDEFAULT, 255, 215, 0, 255)
    drawshadowtext(pSHIP.Crew[i].Health.."%", 470, (35*i)+140, 50, "center", fDEFAULT, 255, 215, 0, 255)
    if pSHIP.Crew[i].Pay==0 then -- do nothing
    elseif math.floor(pSHIP.Crew[i].Pay)==pSHIP.Crew[i].Pay then drawshadowtext(pSHIP.Crew[i].Pay.." gold per day", 530, (35*i)+140, 100, "left", fDEFAULT, 255, 255, 255, 255)
    else drawshadowtext(tostring(2*pSHIP.Crew[i].Pay).." gold per 2 days", 530, (35*i)+140, 100, "left", fDEFAULT, 255, 255, 255, 255) end
    if pSHIP.Crew[i].Role == "Captain" then -- Do nothing, can't be got rid of!
    elseif gGAMEMODE=="sailcrew" then drawshadowtext("X", 640, (35*i)+140, 60, "left", fDEFAULT, 120, 0, 0, 255)
    else drawshadowtext("X", 640, (35*i)+140, 60, "left", fDEFAULT, 255, 255, 255, 255) end
  end
  love.graphics.setColor(0, 0, 0, 255)
  love.graphics.line(106, (35*(#pSHIP.Crew+1))+136, 696, (35*(#pSHIP.Crew+1))+136)
  love.graphics.setColor(255, 215, 0, 255)
  love.graphics.line(105, (35*(#pSHIP.Crew+1))+135, 695, (35*(#pSHIP.Crew+1))+135)
  drawshadowtext(tostring(math.floor((morale/#pSHIP.Crew)+0.5)).."%", 420, (35*(#pSHIP.Crew+1))+140, 50, "center", fDEFAULT, 255, 215, 0, 255)
  drawshadowtext(tostring(math.floor((health/#pSHIP.Crew)+0.5)).."%", 470, (35*(#pSHIP.Crew+1))+140, 50, "center", fDEFAULT, 255, 215, 0, 255)
  if math.floor(wages)==wages then drawshadowtext("Total: "..wages.." per day", 530, (35*(#pSHIP.Crew+1))+140, 170, "left", fDEFAULT, 255, 255, 255, 255)
  else drawshadowtext("Total: "..math.floor(wages).." to "..math.ceil(wages).." per day", 530, (35*(#pSHIP.Crew+1))+140, 170, "left", fDEFAULT, 255, 255, 255, 255) end
  for i = 1, #pSHIP.Crew, 1 do
    if mx>100 and mx<200 and my>(35*i)+140 and my<(35*i)+164 then
      love.graphics.setColor(0, 0, 0, 180)
      love.graphics.rectangle("fill", mx-125, my+25, 250, 110)
      drawshadowtext(pSHIP.Crew[i].Tag, mx-120, my+30, 240, "left", fDEFAULT, 255, 255, 255, 255)
    elseif mx>210 and mx<410 and my>(35*i)+140 and my<(35*i)+164 then
      love.graphics.setColor(0, 0, 0, 180)
      love.graphics.rectangle("fill", mx-125, my+25, 250, 60)
      if pSHIP.Crew[i].Role~="Captain" then drawshadowtext(pSHIP.Crew[i].Name.." has been with the drew for "..pSHIP.Crew[i].Days.." days.", mx-120, my+30, 240, "left", fDEFAULT, 255, 255, 255, 255)
      else drawshadowtext("That's the name your mother gave you, God rest her merry soul.", mx-120, my+30, 240, "left", fDEFAULT, 255, 255, 255, 255) end
    elseif mx>420 and mx<470 and my>(35*i)+140 and my<(35*i)+164 then
      love.graphics.setColor(0, 0, 0, 180)
      love.graphics.rectangle("fill", mx-125, my+25, 250, 135)
      drawshadowtext("Morale is a measure of happiness. Morale declines when a crew member is sick, hungry, unpaid or at sea for long spells without rest. If a crew's collective morale is low, the Captain may face a mutiny!", mx-120, my+30, 240, "left", fDEFAULT, 255, 255, 255, 255)
    elseif mx>470 and mx<520 and my>(35*i)+140 and my<(35*i)+164 then
      love.graphics.setColor(0, 0, 0, 180)
      love.graphics.rectangle("fill", mx-125, my+25, 250, 80)
      drawshadowtext("Health is affected by sickness and hunger. If health reaches 0% the crew member will die.", mx-120, my+30, 240, "left", fDEFAULT, 255, 255, 255, 255)
    elseif mx>530 and mx<630 and my>(35*i)+140 and my<(35*i)+164 and pSHIP.Crew[i].Pay~=0 then
      love.graphics.setColor(0, 0, 0, 180)
      love.graphics.rectangle("fill", mx-125, my+25, 250, 80)
      drawshadowtext("Wages cannot be raised or lowered. If you wish to reduce your wage bill you must let a crew member go.", mx-120, my+30, 240, "left", fDEFAULT, 255, 255, 255, 255)
    elseif mx>640 and mx<700 and my>(35*i)+140 and my<(35*i)+164 and pSHIP.Crew[i].Role~="Captain" then
      love.graphics.setColor(0, 0, 0, 180)
      love.graphics.rectangle("fill", mx-100, my+25, 200, 60)
      if gGAMEMODE=="personnel" then drawshadowtext("Let "..pSHIP.Crew[i].Name.." go?", mx-95, my+30, 190, "center", fDEFAULT, 255, 255, 255, 255)
      else drawshadowtext("Make "..pSHIP.Crew[i].Name.." walk the plank?", mx-95, my+30, 190, "center", fDEFAULT, 255, 255, 255, 255) end
    else end
  end
end

function drawbox(x, y, w, h)
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.draw(iBOX, x, y, 0, w/iBOX:getWidth(), h/iBOX:getHeight(), iBOX:getWidth()/2, iBOX:getHeight()/2)
end

function drawdottedcircle(x, y, r)
  local i
  for i = 0, 360, 1 do
    love.graphics.points(x+(math.cos(math.rad(bearingtoangle(i)))*r), y-(math.sin(math.rad(bearingtoangle(i)))*r))
  end
end

function drawislandmenu()
  local mx, my = love.mouse.getX()/gRATIOX, love.mouse.getY()/gRATIOY
  local menu = {"Shoreline", "Shipwright", "Market", "Tavern", "Personnel", "Course", "Set Sail"}
  local i
  if tISLAND[pSHIP.Docked].Government == "Naval Court" then 
    table.remove(menu, 4)
    table.insert(menu, 4, "Offices")
  end
  drawclock(50, 30)
  for i = 1, #menu, 1 do
    if tISLAND[pSHIP.Docked].Menu[i]=="Yes" then
      drawbox((i*700/#menu)-(350/#menu)+100, 30, 700/#menu, 60)
      if gGAMEMODE==string.lower(menu[i]) then drawshadowtext(menu[i], ((i-1)*700/#menu)+100, 20, 700/#menu, "center", fDEFAULT, 255, 255, 255, 255)
      elseif mx>((i-1)*700/#menu)+100 and mx<(i*700/#menu)+100 and my>0 and my<60 then drawshadowtext(menu[i],  ((i-1)*700/#menu)+100, 20, 700/#menu, "center", fDEFAULT, 255, 215, 0, 255) 
      else drawshadowtext(menu[i],  ((i-1)*700/#menu)+100, 20, 700/#menu, "center", fDEFAULT, 220, 0, 0, 255) end
    end
  end
end

function drawshoreline()
  local mx, my = love.mouse.getX()/gRATIOX, love.mouse.getY()/gRATIOY
  local text = {"Rest for one hour", "Camp until morning", "Forage for food"}
  local i
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.draw(iBACKGROUND[3], 0, 0)
  drawgreybox()
  drawshadowtext("Welcome to "..tISLAND[pSHIP.Docked].Name, 80, 75, 640, "center", fHEADING, 255, 215, 0, 255)
  drawshadowtext("Government:", 80, 200, 100, "right", fDEFAULT, 255, 215, 0, 255)
  drawshadowtext("Population:", 80, 230, 100, "right", fDEFAULT, 255, 215, 0, 255)
  drawshadowtext("Imports:", 80, 260, 100, "right", fDEFAULT, 255, 215, 0, 255)
  drawshadowtext("Exports:", 80, 350, 100, "right", fDEFAULT, 255, 215, 0, 255)
  drawshadowtext(tISLAND[pSHIP.Docked].Tag, 200, 200, 520, "left", fDEFAULT, 255, 255, 255, 255)
  drawshadowtext(tISLAND[pSHIP.Docked].Population, 200, 230, 520, "left", fDEFAULT, 255, 255, 255 ,255)
  drawshadowtext(tISLAND[pSHIP.Docked].Imports, 200, 260, 520, "left", fDEFAULT, 255, 255, 255, 255)
  drawshadowtext(tISLAND[pSHIP.Docked].Exports, 200, 350, 520, "left", fDEFAULT, 255, 255, 255, 255)
  for i = 1, 3, 1 do
    drawbox(506+(i*74), 508, 64, 64)
    love.graphics.setColor(255, 255, 255, 255)
    love.graphics.draw(iICON[18+i], 474+(i*74), 476)
  end
  for i = 1, 3, 1 do
    if mx>474+(i*74) and mx<538+(i*74) and my>476 and my<540 then
      love.graphics.setColor(0, 0, 0, 180)
      love.graphics.rectangle("fill", mx-60, my-50, 120, 30)
      drawshadowtext(text[i], mx-60, my-45, 120, "center", fDEFAULT, 255, 255, 255, 255)
    end
  end
  drawislandmenu()
end

function drawshipwright()
  local mx, my = love.mouse.getX()/gRATIOX, love.mouse.getY()/gRATIOY
  local i, j
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.draw(iBACKGROUND[2], 0, 0)
  drawgreybox()
  drawpurse(685, 107)
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.draw(iICON[7], 80, 107, 0, 64/iICON[7]:getWidth(), 64/iICON[7]:getHeight(), iICON[7]:getWidth()/2, iICON[7]:getHeight()/2)
  drawshadowtext(pSHIP.Ammo.." ammo", 120, 90, 100, "left", fSUBHEADING, 255, 215, 0, 255)
  drawshadowtext("Shipwright", 80, 75, 640, "center", fHEADING, 255, 215, 0, 255)
  drawshadowtext("Upgrades", 0, 150, 400, "center", fSUBHEADING, 255, 215, 0, 255)
  drawshadowtext("Ships", 400, 150, 400, "center", fSUBHEADING, 255, 215, 0, 255)
  j = 190
  for i = 1, #tUPGRADES, 1 do
    if pSHIP.Upgrades[i]=="No" and tISLAND[pSHIP.Docked].Tech>=tUPGRADES[i].Tech and (tUPGRADES[i].Requires=="None" or gethasrequirement(tUPGRADES[i].Requires)) and (tUPGRADES[i].Name~="Repairs" or pSHIP.Wounds>0) then
      drawshadowtext(tUPGRADES[i].Name, 0, j, 190, "right", fDEFAULT, 255, 255, 255, 255)
      if tUPGRADES[i].Name=="Repairs" then drawshadowtext((tUPGRADES[i].Cost*pSHIP.Wounds).." gold", 200, j, 60, "left", fDEFAULT, 255, 215, 0, 255)
      else drawshadowtext(tUPGRADES[i].Cost.." gold", 200, j, 60, "left", fDEFAULT, 255, 215, 0, 255) end
      if (tUPGRADES[i].Name=="Repairs" and pSHIP.Gold>=tUPGRADES[i].Cost*pSHIP.Wounds) or (tUPGRADES[i].Name~="Repairs" and pSHIP.Gold>=tUPGRADES[i].Cost) then
        drawbox(290, j+8, 40, 20)
        if mx>270 and mx<310 and my>j-2 and my<j+18 then drawshadowtext("BUY", 270, j, 40, "center", fDEFAULT, 255, 215, 0, 255)
        else drawshadowtext("BUY", 270, j, 40, "center", fDEFAULT, 255, 255, 255, 255) end
      end
      j = j + 30
    end
  end
  j = 190
  for i = 1, #tBLUEPRINT, 1 do
    if tISLAND[pSHIP.Docked].Tech>=tBLUEPRINT[i].Tech and tISLAND[pSHIP.Docked].Tech<=tBLUEPRINT[i].Tech+2 then
      love.graphics.setColor(255, 255, 255, 255)
      love.graphics.draw(tBLUEPRINT[i].Image[1], 410, j)
      drawshadowtext(tBLUEPRINT[i].Name, 490, j, 300, "left", fDEFAULT, 255, 215, 0, 255)
      drawshadowtext(tBLUEPRINT[i].Price.." gold", 490, j, 300, "right", fDEFAULT, 255, 215, 0, 255)
      drawshadowtext(tBLUEPRINT[i].Tag, 490, j+25, 300, "left", fDEFAULT, 255, 255, 255, 255)
      if pSHIP.Gold>=tBLUEPRINT[i].Price and tBLUEPRINT[i].Type~=pSHIP.Type then
        drawbox(770, j+120, 40, 20)
        if mx>750 and mx<790 and my>j+110 and my<j+130 then drawshadowtext("BUY", 750, j+112, 40, "center", fDEFAULT, 255, 215, 0, 255)
        else drawshadowtext("BUY", 750, j+112, 40, "center", fDEFAULT, 255, 255, 255, 255) end
      end
      j = j + 130
    end
  end
  j = 190
  for i = 1, #tUPGRADES, 1 do
    if pSHIP.Upgrades[i]=="No" and tISLAND[pSHIP.Docked].Tech>=tUPGRADES[i].Tech and (tUPGRADES[i].Requires=="None" or gethasrequirement(tUPGRADES[i].Requires)) and (tUPGRADES[i].Name~="Repairs" or pSHIP.Wounds>0) then
      if mx>0 and mx<260 and my>j and my<j+30 then
        love.graphics.setColor(0, 0, 0, 180)
        love.graphics.rectangle("fill", mx-125, my+25, 250, 30)
        drawshadowtext(tUPGRADES[i].Tag, mx-120, my+30, 240, "center", fDEFAULT, 255, 255, 255, 255)
      end
      j = j + 30
    end
  end
  drawislandmenu()
end

function drawmarket()
  local mx, my = love.mouse.getX()/gRATIOX, love.mouse.getY()/gRATIOY
  local d = pSHIP.Docked
  local i, c
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.draw(iBACKGROUND[7], 0, 0)
  drawgreybox()
  drawshadowtext("Item", 120, 75, 100, "left", fSUBHEADING, 255, 215, 0, 255)
  drawshadowtext("We Pay", 250, 75, 100, "center", fSUBHEADING, 255, 215, 0, 255)
  drawshadowtext("You Pay", 350, 75, 100, "center", fSUBHEADING, 255, 215, 0, 255) 
  drawshadowtext("Cargo", 480, 75, 80, "left", fSUBHEADING, 255, 215, 0, 255) 
  for i = 1, #tGOODS, 1 do
    c = getamountincargo(tGOODS[i].Name)
    drawshadowtext(tGOODS[i].Name, 120, (i*50)+75, 100, "left", fSUBHEADING, 255, 255, 255, 255)
    if tISLAND[d].Buy[i]~=0 then drawshadowtext(tISLAND[d].Buy[i].."g", 280, (i*50)+75, 80, "left", fSUBHEADING, 255, 255, 255, 255)
    else drawshadowtext("X", 280, (i*50)+75, 80, "left", fSUBHEADING, 255, 255, 255, 255) end
    if tISLAND[d].Sell[i]~=0 then drawshadowtext(tISLAND[d].Sell[i].."g", 380, (i*50)+75, 80, "left", fSUBHEADING, 255, 255, 255, 255)
    else drawshadowtext("X", 380, (i*50)+75, 80, "left", fSUBHEADING, 255, 255, 255, 255) end
    if c==1 then drawshadowtext("1 crate", 480, (i*50)+75, 100, "left", fSUBHEADING, 255, 255, 255, 255)
    elseif c>=2 then drawshadowtext(c.." crates", 480, (i*50)+75, 100, "left", fSUBHEADING, 255, 255, 255, 255)
    else end
    drawbox(595, (i*50)+90, 30, 30)
    if c>=1 then drawshadowtext("-", 580, (i*50)+82, 30, "center", fDEFAULT, 255, 255, 255, 255) end
    drawbox(645, (i*50)+90, 30, 30)
    if pSHIP.Gold>=tISLAND[d].Sell[i] and tISLAND[d].Sell[i]>0 and gettablesize(pSHIP.Cargo)<getcapacity(pSHIP) then drawshadowtext("+", 630, (i*50)+82, 30, "center", fDEFAULT, 255, 255, 255, 255) end
  end
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.draw(iICON[6], 220, ((#tGOODS+1)*50)+75)
  love.graphics.draw(iICON[5], 470, ((#tGOODS+1)*50)+75)
  drawshadowtext(math.ceil(pSHIP.Food).." food", 330, ((#tGOODS+1)*50)+90, 130, "left", fSUBHEADING, 255, 215, 0, 255)
  drawshadowtext(getcapacity(pSHIP)-gettablesize(pSHIP.Cargo), 470, ((#tGOODS+1)*50)+90, 64, "center", fSUBHEADING, 255, 215, 0, 255)
  drawshadowtext("free cargo slots", 534, ((#tGOODS+1)*50)+90, 146, "left", fSUBHEADING, 255, 215, 0, 255)
  drawpurse(120, ((#tGOODS+1)*50)+107)
  drawbox(305, ((#tGOODS+1)*50)+107, 30, 30)
  if math.ceil(pSHIP.Food)<100 then drawshadowtext("+", 290, ((#tGOODS+1)*50)+99, 30, "center", fDEFAULT, 255, 255, 255, 255) end
  for i = 1, #tGOODS, 1 do
    if mx>120 and mx<220 and my>(i*50)+75 and my<(i*50)+107 then
      love.graphics.setColor(0, 0, 0, 180)
      love.graphics.rectangle("fill", mx-120, my+25, 240, 60)
      drawshadowtext(tGOODS[i].Tag, mx-115, my+30, 230, "left", fDEFAULT, 255, 215, 0, 255)
    end
  end
  if mx>290 and mx<320 and my>((#tGOODS+1)*50)+92 and my<((#tGOODS+1)*50)+122 then
    love.graphics.setColor(0, 0, 0, 180)
    love.graphics.rectangle("fill", mx-120, my-50, 240, 30)
    drawshadowtext("Food costs 1 gold per unit.", mx-115, my-45, 230, "left", fDEFAULT, 255, 215, 0, 255)
  end
  drawislandmenu()
end

function drawpurse(x,y)
  local text = math.floor(pSHIP.Gold)
  if text<10000 then text = tostring(text.." gold") end
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.draw(iICON[18], x-104, y-32)
  drawshadowtext(text, x-30, y-17, 130, "left", fSUBHEADING, 255, 215, 0, 255)
end

function drawtavern()
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.draw(iBACKGROUND[4], 0, 0)
  drawgreybox()
  drawpurse(685, 107)
  drawshadowtext("Tavern", 80, 75, 640, "center", fHEADING, 255, 215, 0, 255)
  drawbar(200, 260)
  drawsalemaps(600, 260)
  drawhirecrew(200, 490)
  drawislandmenu()
end

function drawoffices()
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.draw(iBACKGROUND[8], 0, 0)  
  drawgreybox()
  drawpurse(685, 107)
  drawshadowtext("Offices", 80, 75, 640, "center", fHEADING, 255, 215, 0, 255)
  drawsalemaps(600, 260)
  drawhirecrew(200, 490)
  drawcartographer(600, 490)
  drawislandmenu()
end

function drawbar(x,y)
  local mx, my = love.mouse.getX()/gRATIOX, love.mouse.getY()/gRATIOY
  local menu = {"Round of Grog:", "Basic room:", "Luxury room:"}
  local cost = {math.ceil(#pSHIP.Crew/2), 5, 10}
  local tag = {"Improves crew morale.", "Rest until morning. Small recovery of morale and health.", "Rest until morning. Large recovery of morale and health."}
  local i
  drawshadowtext("Services", x-190, y-105, 380, "center", fSUBHEADING, 255, 215, 0, 255)
  for i = 1, #menu, 1 do
    drawshadowtext(menu[i], x-180, y-85+(i*30), 195, "right", fDEFAULT, 255, 255, 255, 255)
    drawshadowtext(cost[i].." gold", x+25, y-85+(i*30), 55, "left", fDEFAULT, 255, 215, 0, 255)
    if pSHIP.Gold>=cost[i] then
      drawbox(x+100, y-78+(i*30), 40, 20)
      if mx>x+80 and mx<x+120 and my>y-88+(i*30) and my<y-68+(i*30) then drawshadowtext("BUY", x+80, y-85+(i*30), 40, "center", fDEFAULT, 255, 215, 0, 255)
      else drawshadowtext("BUY", x+80, y-85+(i*30), 40, "center", fDEFAULT, 255, 255, 255, 255) end
    end
  end
  for i = 1, #menu, 1 do
    if mx>x-180 and mx<x+80 and my>y-85+(i*30) and my<y-66+(i*30) then
      love.graphics.setColor(0, 0, 0, 180)
      love.graphics.rectangle("fill", mx-100, my+25, 200, 60)
      drawshadowtext(tag[i], mx-95, my+35, 190, "left", fDEFAULT, 255, 255, 255, 255)
    end
  end
end

function drawsalemaps(x,y)
  local mx, my = love.mouse.getX()/gRATIOX, love.mouse.getY()/gRATIOY
  local i
  if next(tISLAND[pSHIP.Docked].Maps)~=nil then
    drawshadowtext("Maps for Sale", x-190, y-105, 380, "center", fSUBHEADING, 255, 215, 0, 255)
    for i = 1, #tISLAND[pSHIP.Docked].Maps, 1 do
      drawshadowtext("Map to "..tISLAND[pSHIP.Docked].Maps[i].Name..":", x-180, y-85+(i*30), 195, "right", fDEFAULT, 255, 255, 255, 255)
      drawshadowtext(tISLAND[pSHIP.Docked].Maps[i].Cost.." gold", x+25, y-85+(i*30), 55, "left", fDEFAULT, 255, 215, 0, 255)
      drawbox(x+100, y-78+(i*30), 40, 20)
      if mx>x+80 and mx<x+120 and my>y-88+(i*30) and my<y-68+(i*30) then drawshadowtext("BUY", x+80, y-85+(i*30), 40, "center", fDEFAULT, 255, 215, 0, 255)
      else drawshadowtext("BUY", x+80, y-85+(i*30), 40, "center", fDEFAULT, 255, 255, 255, 255) end
    end
    for i = 1, #tISLAND[pSHIP.Docked].Maps, 1 do
      if mx>x-180 and mx<x+80 and my>y-85+(i*30) and my<y-66+(i*30) then 
        love.graphics.setColor(0, 0, 0, 180)
        love.graphics.rectangle("fill", mx-80, my+25, 160, 110)
        drawshadowtext("This map leads to a genuine island but you cannot know what kind of island before purchase.", mx-75, my+35, 150, "left", fDEFAULT, 255, 255, 255, 255)
      end
    end
  else drawshadowtext("There are no maps for sale at this time.", x-180, y-16, 360, "center", fSUBHEADING, 255, 255, 255, 255) end
end

function drawhirecrew(x,y)
  local mx, my = love.mouse.getX()/gRATIOX, love.mouse.getY()/gRATIOY
  local i
  if next(tISLAND[pSHIP.Docked].Crew)~=nil then
    drawshadowtext("Crew for Hire", x-190, y-105, 380, "center", fSUBHEADING, 255, 215, 0, 255)
    for i = 1, #tISLAND[pSHIP.Docked].Crew, 1 do
      drawshadowtext(tISLAND[pSHIP.Docked].Crew[i].Role, x-180, y-85+(i*30), 90, "right", fDEFAULT, 255, 215, 0, 255)
      drawshadowtext(tISLAND[pSHIP.Docked].Crew[i].Name, x-80, y-85+(i*30), 190, "left", fDEFAULT, 255, 255, 255, 255)
      if (not gethasrole(tISLAND[pSHIP.Docked].Crew[i].Role) or tISLAND[pSHIP.Docked].Crew[i].Unique=="No") then
        drawbox(x+140, y-78+(i*30), 50, 20)
        if mx>x+115 and mx<x+165 and my>y-88+(i*30) and my<y-68+(i*30) then drawshadowtext("HIRE", x+120, y-85+(i*30), 40, "center", fDEFAULT, 255, 215, 0, 255)
        else drawshadowtext("HIRE", x+120, y-85+(i*30), 40, "center", fDEFAULT, 255, 255, 255, 255) end
      end
    end
    for i = 1, #tISLAND[pSHIP.Docked].Crew, 1 do
      if mx>x-180 and mx<x+110 and my>y-85+(i*30) and my<y-66+(i*30) then
        love.graphics.setColor(0, 0, 0, 180)
        love.graphics.rectangle("fill", mx-125, my-140, 250, 110)
        drawshadowtext(tISLAND[pSHIP.Docked].Crew[i].Tag, mx-120, my-135, 240, "left", fDEFAULT, 255, 255, 255, 255)
      end
    end
  else drawshadowtext("There is no one available for hire.", x-180, y-16, 360, "center", fSUBHEADING, 255, 255, 255, 255) end
end

function drawcartographer(x,y)
  local mx, my = love.mouse.getX()/gRATIOX, love.mouse.getY()/gRATIOY
  drawshadowtext("Cartographer's Office", x-190, y-105, 380, "center", fSUBHEADING, 255, 215, 0, 255)
  if pSHIP.Cartographer > 0 then
    drawshadowtext("You have discovered "..pSHIP.Cartographer.." islands.", x-180, y-56, 360, "center", fSUBHEADING, 255, 255, 255, 255)
    drawshadowtext("You are owed "..(pSHIP.Cartographer*25).." gold.", x-180, y-16, 360, "center", fSUBHEADING, 255, 215, 0, 255)
    drawbox(x, y+36, 60, 20)
    if mx>x-30 and mx<x+30 and my>y+26 and my<y+46 then drawshadowtext("CLAIM", x-30, y+28, 60, "center", fDEFAULT, 255, 215, 0, 255)
    else drawshadowtext("CLAIM", x-30, y+28, 60, "center", fDEFAULT, 255, 255, 255, 255) end
  elseif pSHIP.Cartographer == -1 then
    drawshadowtext("Registration Fee", x-180, y-56, 360, "center", fSUBHEADING, 255, 255, 255, 255)
    drawshadowtext("100 gold", x-180, y-16, 360, "center", fSUBHEADING, 255, 215, 0, 255)
    if pSHIP.Gold>=100 then 
      drawbox(x, y+36, 40, 20)
      if mx>x-20 and mx<x+20 and my>y+26 and my<y+46 then drawshadowtext("PAY", x-20, y+28, 40, "center", fDEFAULT, 255, 215, 0, 255)
      else drawshadowtext("PAY", x-20, y+28, 40, "center", fDEFAULT, 255, 255, 255, 255) end
    end
  else drawshadowtext("You have no wages to claim.", x-180, y-16, 360, "center", fSUBHEADING, 255, 255, 255, 255) end
end

function drawpersonnel()
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.draw(iBACKGROUND[5], 0, 0)
  drawgreybox()
  drawcrew()
  drawislandmenu()
end

function drawcourse()
  drawmap()
  drawislandmenu()
end

function drawending()
  drawgame()
end

function drawgameover()
  drawgame()
  drawgreybox()
  drawshadowtext("Game over", 0, 236, 800, "center", fTITLE, 255, 215, 0, 255)
  drawshadowtext("Press space bar", 0, 332, 800, "center", fSUBHEADING, 255, 255, 255, 255)
end

function drawfloatytext()
  local text = ""
  local width, wrappedtext
  local i
  if next(gFLOATTEXT)~=nil then
    for i = 1, #gFLOATTEXT, 1 do
      if i==1 then text = text..gFLOATTEXT[i].Text
      else text = text.."\n"..gFLOATTEXT[i].Text end
    end
    width, wrappedtext = fSUBHEADING:getWrap(text, 390)
    drawbox(400, 120+(#wrappedtext*20), 400, #wrappedtext*40)
    drawshadowtext(text, 205, 125, 390, "left", fSUBHEADING, 255, 255, 255, 255)
  end
end

function love.mousemoved(x, y, dx, dy, istouch)
  x, y = x/gRATIOX, y/gRATIOY
  dx, dy = dx/gRATIOX, dy/gRATIOY
  if _G["moved"..gGAMEMODE]~=nil then
    _G["moved"..gGAMEMODE](x, y, dx, dy)
  end
end

function movedcourse(x, y, dx, dy)
  movedmap(x, y, dx, dy)
end

function movedsailmap(x, y, dx, dy)
  movedmap(x, y, dx, dy)
end

function movedmap(x, y, dx, dy)
  if love.mouse.isDown(1, 2, 3) then
    gMAPX = gMAPX - (dx/tSETTINGS.MapZoom)
    gMAPY = gMAPY + (dy/tSETTINGS.MapZoom)
  end
end

function love.mousepressed(x, y, button, istouch, presses)  -- Record any click, based on gamemode
  x,y = x/gRATIOX, y/gRATIOY
  if _G["click"..gGAMEMODE]~=nil then
    _G["click"..gGAMEMODE](x,y)
    playsound(17)
  end
end

function clickmenu(x,y)
  if x>40 and x<252 and y>300 and y<364 then    -- clicked newgame
    initialise()
    gGAMEMODE = "choosenames"
    gTYPING = false
    gTYPEDTEXT = ""
  elseif x>580 and x<680 and y>300 and y<364 then  -- clicked quit
    quitgame()
  elseif x>266 and x<532 and y>400 and y<500 then   -- clicked continue
    if gGAMEMODE ~= gOLDGAMEMODE then
      gGAMEMODE = gOLDGAMEMODE
      setsail()
    elseif love.filesystem.getInfo("save_player.txt", "file") then loadgame()
    else end
  elseif x>0 and x<50 and y>550 and y<600 then   -- toggle music
    if tSETTINGS.Music=="Yes" then
      tSETTINGS.Music="No"
      musicstop()
      writesingletable("settings.txt", tSETTINGS)
    else
      tSETTINGS.Music="Yes"
      musicplay(1)
      writesingletable("settings.txt", tSETTINGS)
    end
  elseif x>750 and x<800 and y>550 and y<600 then   -- toggle sound
    if tSETTINGS.Sound=="Yes" then
      playsound(17)
      tSETTINGS.Sound="No"
      playwind("Off")
      playwaves("Off")
      writesingletable("settings.txt", tSETTINGS)
    else
      tSETTINGS.Sound="Yes"
      writesingletable("settings.txt", tSETTINGS)
    end
  else end
end

function clickchoosenames(x,y)
  if x>0 and x<64 and y>0 and y<64 then startmenu()
  elseif x>275 and x<800 and y>200 and y<264 then
    if gTYPING ~= true then
      gTYPING = true
      gTYPEDTEXT = ""
      gSELECT = 1
    else 
      pSHIP.PlayerName = gTYPEDTEXT
      pSHIP.Crew[1].Name = gTYPEDTEXT
      gTYPING = false
    end
  elseif x>275 and x<800 and y>340 and y<404 then
    if gTYPING ~= true then
      gTYPING = true
      gTYPEDTEXT = ""
      gSELECT = 2
    else 
      pSHIP.Name = gTYPEDTEXT
      gTYPING = false
    end
  elseif x>650 and x<725 and y>270 and y<294 then pSHIP.PlayerName = tCAPNAME[math.random(#tCAPNAME)]
  elseif x>650 and x<725 and y>410 and y<434 then pSHIP.Name = tSHIPNAME[math.random(#tSHIPNAME)]
  elseif x>200 and x<600 and y>490 and y<590 then 
    gSELECT = 1
    love.mouse.setVisible(false)
    gGAMEMODE = "newgame"
  else end
end

function clicksailmap(x,y)
  clickmap(x,y)
end

function clickmap(x,y)
  local i
  if x>660 and x<690 and y>460 and y<490 then 
    tSETTINGS.MapZoom = math.min(tSETTINGS.MapZoom+0.01, 0.25)
  elseif x>660 and x<690 and y>490 and y<520 then 
    tSETTINGS.MapZoom = math.max(tSETTINGS.MapZoom-0.01, 0.05)
  elseif x>80 and x<720 and y>60 and y<540 then
    if (love.keyboard.isDown("lctrl") or love.keyboard.isDown("rctrl")) and gethasrole("Navigator") then
      pSHIP.TX, pSHIP.TY = gMAPX+((x-400)/tSETTINGS.MapZoom), gMAPY+((300-y)/tSETTINGS.MapZoom)
    end
  else end
end

function clickislandmenu(x,y)
  local menu = {"Shoreline", "Shipwright", "Market", "Tavern", "Personnel", "Course", "Set Sail"}
  if tISLAND[pSHIP.Docked].Government == "Naval Court" then 
    table.remove(menu, 4)
    table.insert(menu, 4, "Offices")
  end
  for i = 1, #menu, 1 do
    if x>((i-1)*700/#menu)+100 and x<(i*700/#menu)+100 and y>0 and y<60 and tISLAND[pSHIP.Docked].Menu[i]=="Yes" then
      if gGAMEMODE~=string.lower(menu[i]) and menu[i]~="Set Sail" then
        gSELECT = 1
        gGAMEMODE = string.lower(menu[i])
        if menu[i]=="Course" then gMAPX, gMAPY = pSHIP.X, pSHIP.Y
        elseif menu[i]=="Shipwright" then playsound(27)
        elseif menu[i]=="Market" then playsound(24)
        elseif menu[i]=="Offices" then playsound(25)
        elseif menu[i]=="Tavern" then playsound(26)
        else end
      elseif menu[i]=="Set Sail" then 
        savegame()
        setsail()
      else end
    end
  end
end  

function clickshoreline(x,y)
  if x>548 and x<612 and y>476 and y<540 then 
    processhour()
    gFADE = 255
  elseif x>622 and x<686 and y>476 and y<540 then
    processhour()
    while tCLOCK.Hour~=6 do
      processhour()
    end
    gFADE = 255
  elseif x>696 and x<760 and y>476 and y<540 then forage()
  else end
  clickislandmenu(x,y)
end

function clickshipwright(x,y)
  local i, j
  j = 190
  for i = 1, #tUPGRADES, 1 do
    if pSHIP.Upgrades[i]=="No" and tISLAND[pSHIP.Docked].Tech>=tUPGRADES[i].Tech and (tUPGRADES[i].Requires=="None" or gethasrequirement(tUPGRADES[i].Requires)) and (tUPGRADES[i].Name~="Repairs" or pSHIP.Wounds>0) then
      if x>270 and x<310 and y>j-8 and y<j+18 then
        if tUPGRADES[i].Name=="Repairs" and pSHIP.Gold>=tUPGRADES[i].Cost*pSHIP.Wounds then
          pSHIP.Gold = pSHIP.Gold - tUPGRADES[i].Cost*pSHIP.Wounds
          playsound(8)
          pSHIP.Wounds = 0
          addfloatytext("It takes an hour for the shipwright to finish repairing the ship.")
          processhour()
        elseif tUPGRADES[i].Name=="Cannonball" and pSHIP.Gold>=tUPGRADES[i].Cost then
          pSHIP.Gold = pSHIP.Gold - tUPGRADES[i].Cost
          playsound(8)
          pSHIP.Ammo = pSHIP.Ammo + 1
        elseif tUPGRADES[i].Name~="Repairs" and tUPGRADES[i].Name~="Cannonball" and pSHIP.Gold>=tUPGRADES[i].Cost then
          pSHIP.Gold = pSHIP.Gold - tUPGRADES[i].Cost
          playsound(8)
          pSHIP.Upgrades[i] = "Yes"
          addfloatytext(tUPGRADES[i].Name.." upgrade purchased.")
        else end
      end
      j =j + 30
    end
  end
  j = 190
  for i = 1, #tBLUEPRINT, 1 do
    if tISLAND[pSHIP.Docked].Tech>=tBLUEPRINT[i].Tech and tISLAND[pSHIP.Docked].Tech<=tBLUEPRINT[i].Tech+2 then
      if x>750 and x<790 and y>j+110 and y<j+130 and pSHIP.Gold>=tBLUEPRINT[i].Price and tBLUEPRINT[i].Type~=pSHIP.Type then
        pSHIP.Gold = pSHIP.Gold - tBLUEPRINT[i].Price
        purchaseship(i)
        playsound(8)
      end
      j = j + 130
    end
  end
  clickislandmenu(x,y)
end

function clickmarket(x,y)
  local i
  for i = 1, #tGOODS, 1 do
    if x>580 and x<610 and y>(i*50)+75 and y<(i*50)+105 and getamountincargo(tGOODS[i].Name)>0 and tISLAND[pSHIP.Docked].Buy[i]>0 then
      pSHIP.Gold = pSHIP.Gold + tISLAND[pSHIP.Docked].Buy[i]
      if math.random(40)==1 then 
        tISLAND[pSHIP.Docked].Buy[i] = tISLAND[pSHIP.Docked].Buy[i] - 1
        tISLAND[pSHIP.Docked].Sell[i] = tISLAND[pSHIP.Docked].Sell[i] - 1
      end
      removecargo(tGOODS[i].Name)
      playsound(8, "Yes")
    elseif x>630 and x<660 and y>(i*50)+75 and y<(i*50)+105 and pSHIP.Gold>=tISLAND[pSHIP.Docked].Sell[i] and gettablesize(pSHIP.Cargo)<getcapacity(pSHIP) then
      table.insert(pSHIP.Cargo, tGOODS[i].Name)
      pSHIP.Gold = pSHIP.Gold - tISLAND[pSHIP.Docked].Sell[i]
      if math.random(40)==1 then 
        tISLAND[pSHIP.Docked].Sell[i] = tISLAND[pSHIP.Docked].Sell[i] + 1
        tISLAND[pSHIP.Docked].Buy[i] = tISLAND[pSHIP.Docked].Buy[i] + 1
      end
      playsound(8, "Yes")
    else end
  end
  if x>290 and x<320 and y>((#tGOODS+1)*50)+92 and y<((#tGOODS+1)*50)+122 and math.ceil(pSHIP.Food)<100 and pSHIP.Gold>=1 then
    pSHIP.Gold = pSHIP.Gold - 1
    pSHIP.Food = pSHIP.Food + 1
    playsound(8, "Yes")
  end
  clickislandmenu(x,y)
end

function clicktavern(x,y)
  local i, j, rest
  if tCLOCK.Hour>=6 then rest = 30-tCLOCK.Hour
  else rest = 6-tCLOCK.Hour end 
  if x>280 and x<320 and y>202 and y<222 and pSHIP.Gold>=math.ceil(#pSHIP.Crew/2) then
    pSHIP.Gold = pSHIP.Gold - math.ceil(#pSHIP.Crew/2)
    for i = 1, #pSHIP.Crew, 1 do
      pSHIP.Crew[i].Morale = math.min(pSHIP.Crew[i].Morale+4+math.random(10), 100)
    end
    playsound(8)
  elseif x>280 and x<320 and y>232 and y<252 and pSHIP.Gold>=5 then
    pSHIP.Gold = pSHIP.Gold - 5
    playsound(8)
    while rest > 0 do
      for i = 1, #pSHIP.Crew, 1 do
        pSHIP.Crew[i].Morale = math.min(pSHIP.Crew[i].Morale+(math.random(4)-1), 100)
        pSHIP.Crew[i].Health = math.min(pSHIP.Crew[i].Health+(math.random(5)-1), 100)
      end
      processhour()
      rest = rest - 1
    end
  elseif x>280 and x<320 and y>262 and y<282 and pSHIP.Gold>=10 then
    pSHIP.Gold = pSHIP.Gold - 10
    playsound(8)
    while rest > 0 do
      for i = 1, #pSHIP.Crew, 1 do
        pSHIP.Crew[i].Morale = math.min(pSHIP.Crew[i].Morale+(math.random(8)-1), 100)
        pSHIP.Crew[i].Health = math.min(pSHIP.Crew[i].Health+(math.random(10)-1), 100)
      end
      processhour()
      rest = rest - 1
    end
  else end
  clicksalemaps(x,y)
  clickhirecrew(x,y)
  clickislandmenu(x,y)
end

function clickoffices(x,y)
  if x>570 and x<630 and y>516 and y<536 and pSHIP.Cartographer==-1 and pSHIP.Gold>=100 then
    pSHIP.Gold = pSHIP.Gold - 100
    playsound(8)
    addfloatytext("Cartographer: You may now claim wages for discovering islands.")
    pSHIP.Cartographer = 0
  elseif x>575 and x<625 and y>516 and y<536 and pSHIP.Cartographer>=1 then
    pSHIP.Gold = pSHIP.Gold + (pSHIP.Cartographer*25)
    addfloatytext("Cartographer: "..(pSHIP.Cartographer*25).." gold received.")
    pSHIP.Cartographer = 0
    playsound(8)
  else end
  clicksalemaps(x,y)
  clickhirecrew(x,y)
  clickislandmenu(x,y)
end

function clicksalemaps(x,y)
  local i
  if next(tISLAND[pSHIP.Docked].Maps)~=nil then
    for i = #tISLAND[pSHIP.Docked].Maps, 1, -1 do
      if x>680 and x<720 and y>172+(i*30) and y<192+(i*30) and pSHIP.Gold>=tISLAND[pSHIP.Docked].Maps[i].Cost then
        pSHIP.Gold = pSHIP.Gold - tISLAND[pSHIP.Docked].Maps[i].Cost
        tISLAND[tISLAND[pSHIP.Docked].Maps[i].Index].Discovered = "Yes"
        table.remove(tISLAND[pSHIP.Docked].Maps, i)
        playsound(8)
      end
    end
  end
end

function clickhirecrew(x,y)
  local i
  if next(tISLAND[pSHIP.Docked].Crew)~=nil then
    for i = #tISLAND[pSHIP.Docked].Crew, 1, -1 do
      if x>315 and x<365 and y>402+(i*30) and y<422+(i*30) and (not gethasrole(tISLAND[pSHIP.Docked].Crew[i].Role) or tISLAND[pSHIP.Docked].Crew[i].Unique=="No") then
        table.insert(pSHIP.Crew, deepcopy(tISLAND[pSHIP.Docked].Crew[i]))
        table.remove(tISLAND[pSHIP.Docked].Crew, i)
      end
    end
  end
end

function clickpersonnel(x,y)
  local i
  for i = #pSHIP.Crew, 1, -1 do
    if x>640 and x<660 and y>(35*i)+140 and y<(35*i)+164 then
      table.insert(tISLAND[pSHIP.Docked].Crew, deepcopy(pSHIP.Crew[i]))
      table.remove(pSHIP.Crew, i)
    end
  end
  clickislandmenu(x,y)
end

function clickcourse(x,y)
  clickmap(x,y)
  clickislandmenu(x,y)
end

function love.mousereleased(x, y, button, istouch, presses)
  x,y = x/gRATIOX, y/gRATIOY
  if _G["release"..gGAMEMODE]~=nil then
    _G["release"..gGAMEMODE](x,y)
  end
end

function love.textinput(text)
  if gTYPING==true then
    gTYPEDTEXT = gTYPEDTEXT..text
    if string.len(gTYPEDTEXT)>20 then gTYPEDTEXT=string.sub(gTYPEDTEXT,1,20) end
  end
end

function love.keypressed(key)  -- Record any keypress, based on gamemode
  if _G["key"..gGAMEMODE]~=nil then 
    _G["key"..gGAMEMODE](key)
  end
end  

function keymenu(key)
  if key=="escape" then
    if gGAMEMODE~=gOLDGAMEMODE then setsail()
    else quitgame() end
  end
end

function keychoosenames(key)
  if key=="escape" then 
    if gTYPING==true then gTYPING=false
    else startmenu() end
  elseif key=="return" then 
    if gTYPING==true then 
      if gSELECT==1 then 
        pSHIP.PlayerName = gTYPEDTEXT
        pSHIP.Crew[1].Name = gTYPEDTEXT
      elseif gSELECT==2 then pSHIP.Name = gTYPEDTEXT
      else end
      gTYPING = false
    else setsail() end
  elseif key=="backspace" and gTYPING==true and string.len(gTYPEDTEXT)>=1 then gTYPEDTEXT=string.sub(gTYPEDTEXT,1,string.len(gTYPEDTEXT)-1)
  else end
end

function keyhelp(key)
  gGAMEMODE = "sail"
  playwaves("On")
  playwind("On")
end

function keysail(key)
  local h = gethealth(pSHIP)
  if key=="escape" then
    gOLDGAMEMODE = "sail"
    playwind("Off")
    playwaves("Off")
    startmenu()
  elseif key=="up" and pSHIP.Anchor=="Up" then
    if pSHIP.Sail=="Oars only" then 
      pSHIP.Sail="Half sail"
      playsound(4)
    elseif pSHIP.Sail=="Half sail" then
      pSHIP.Sail="Full sail" 
      playsound(4)
    else end
  elseif key=="down" and pSHIP.Anchor=="Up" then
    if pSHIP.Sail=="Full sail" then 
      pSHIP.Sail="Half sail"
      playsound(5)
    elseif pSHIP.Sail=="Half sail" then
      pSHIP.Sail="Oars only"
      playsound(5)
    else end
  elseif (key=="left" or key=="right") and pSHIP.Anchor=="Up" then
    playsound(17+math.random(4))
  elseif key=="a" and h>0 then
    if pSHIP.Anchor=="Down" then 
      playsound(3)
      pSHIP.Anchor="Up"
    elseif pSHIP.Anchor=="Up" and pSHIP.Sail=="Oars only" then 
      playsound(2)
      pSHIP.Anchor="Down"
    else end
  elseif key=="c" then
    if tSETTINGS.Compass=="Yes" then tSETTINGS.Compass="No"
    else tSETTINGS.Compass="Yes" end
  elseif key=="g" and pSHIP.Docked == 0 and getislanddocked(pSHIP.X, pSHIP.Y)~=0 and pSHIP.Anchor=="Down" and getplayerwelcome(getislanddocked(pSHIP.X, pSHIP.Y)) then
    gGAMEMODE="shoreline"
    gFADE = 255
    pSHIP.Docked = getislanddocked(pSHIP.X, pSHIP.Y)
    playsound(20)
    love.mouse.setVisible(true)
    playwind("Off")
    playwaves("Off")
    savegame()
    calculateimportsexports()
  elseif key=="h" then 
    gGAMEMODE = "help"
    playwind("Off")
    playwaves("Off")
  elseif key=="i" then
    if tSETTINGS.Info=="Yes" then tSETTINGS.Info="No"
    else tSETTINGS.Info="Yes" end
  elseif key=="m" and gethasrole("Navigator") then 
    gGAMEMODE="sailmap" 
    gMAPX, gMAPY = pSHIP.X, pSHIP.Y 
    love.mouse.setVisible(true)
    playwind("Off")
    playwaves("Off")
  elseif key=="n" then
    if tSETTINGS.Names=="Yes" then tSETTINGS.Names="No"
    else tSETTINGS.Names="Yes" end
  elseif key=="p" then
    gGAMEMODE="sailcrew"
    love.mouse.setVisible(true)
    playwind("Off")
    playwaves("Off")
  elseif key=="w" then
    if tSETTINGS.Wind=="Yes" then tSETTINGS.Wind="No"
    else tSETTINGS.Wind="Yes" end
  elseif key=="x" and h>0 and pSHIP.Ammo > 0 and pSHIP.Firing==0 and pSHIP.Reloading==0 then
    pSHIP.Cannons = "Right"
    pSHIP.Firing = tSETTINGS.FPS
    playsound(7)
  elseif key=="z" and h>0 and pSHIP.Ammo > 0 and pSHIP.Firing==0 and pSHIP.Reloading==0 then
    pSHIP.Cannons = "Left"
    pSHIP.Firing = tSETTINGS.FPS 
    playsound(7)
  else end
end

function keysailmap(key)
  if key=="lctrl" or key=="rctrl" then -- Do nothing, ctrl-click
  elseif key=="space" then pSHIP.TX, pSHIP.TY = nil, nil
  else
    gGAMEMODE = "sail"
    love.mouse.setVisible(false)
    playwind("On")
    playwaves("On")
  end
end

function keysailcrew(key)
  gGAMEMODE = "sail"
  love.mouse.setVisible(false)
  playwind("On")
  playwaves("On")
end

function keyislandmenu(key)
  if key == "escape" then 
    if gGAMEMODE~="shoreline" then gGAMEMODE = "shoreline"
    else setsail() end
  elseif key == "1" then gGAMEMODE = "shoreline"
  elseif (key=="m" or key=="5") and gGAMEMODE ~= "course" then 
    gGAMEMODE = "course"
    gMAPX, gMAPY = pSHIP.X, pSHIP.Y
  elseif key=="3" and tISLAND[pSHIP.Docked].Menu[3]=="Yes" and gGAMEMODE ~= "market" then
    gGAMEMODE = "market"
    playsound(24)
  elseif key=="4" and tISLAND[pSHIP.Docked].Government=="Naval Court" and gGAMEMODE ~= "offices" then
    gGAMEMODE = "offices"
    playsound(25)
  elseif key=="p" or key=="6" then gGAMEMODE = "personnel"
  elseif key=="s" or key=="7" then setsail()
  elseif key=="4" and tISLAND[pSHIP.Docked].Government~="Naval Court" and tISLAND[pSHIP.Docked].Menu[4]=="Yes" and gGAMEMODE ~= "tavern" then
    gGAMEMODE = "tavern"
    playsound(26)
  elseif key=="2" and tISLAND[pSHIP.Docked].Menu[2]=="Yes" and gGAMEMODE ~= "shipwright" then
    gGAMEMODE = "shipwright"
    playsound(27)
  else end
end

function keyshoreline(key)
  keyislandmenu(key)
end

function keyshipwright(key)
  keyislandmenu(key)
end

function keymarket(key)
  keyislandmenu(key)
end

function keytavern(key)
  keyislandmenu(key)
end

function keyoffices(key)
  keyislandmenu(key)
end

function keypersonnel(key)
  keyislandmenu(key)
end

function keycourse(key)
  if key=="space" then pSHIP.TX, pSHIP.TY = nil, nil end
  keyislandmenu(key)
end

function keygameover(key)
  if key=="space" then startmenu() end
end

function getlighting()
  local r = {  0,  0,  0,  0,  0,  0,  0,255,255,255,255,255,255,255,255,255,255,255,255,255,  0,  0,  0,  0}
  local g = {  0,  0,  0,  0,  0,  0,  0,225,225,225,225,225,225,225,225,225,225,225,225,225,  0,  0,  0,  0}
  local b = { 10, 10, 10, 10, 10, 10, 10,200,200,200,200,200,200,200,200,200,200,200,200,200, 10, 10, 10, 10}
  local a = {180,180,150,150,100,50,  0, 25, 50, 50, 50,  50, 50, 50, 50, 50, 50,  50, 25,  0, 50,100,150,150}
  local h1, m, h2 = tCLOCK.Hour+1, tCLOCK.Min, tCLOCK.Hour+2
  local m1, m2 = (59-m)/59, m/59
  if h2>=25 then h2 = 1 end
  return (r[h1]*m1)+(r[h2]*m2), (g[h1]*m1)+(g[h2]*m2), (b[h1]*m1)+(b[h2]*m2), (a[h1]*m1)+(a[h2]*m2)
end

function gettablesize(t)
  local size = 0
  if next(t)~=nil then size = #t end
  return size
end

function getcapacity(ship)
  local c = ship.Capacity
  if ship.Upgrades[4]=="Yes" then c = c + 1 end
  if ship.Upgrades[5]=="Yes" then c = c + 1 end
  if ship.Upgrades[8]=="Yes" then c = c + 1 end
  return c
end

function getcannonaccuracy(ship)
  local acc = 7
  if ship.Upgrades[9]=="Yes" then acc = acc - 2 end
  if ship.Upgrades[16]=="Yes" then acc = acc - 2 end
  if ship.Upgrades[23]=="Yes" then acc = acc - 2 end
end

function getcannonpower(ship)
  local power = ship.Power
  if ship.Upgrades[12]=="Yes" then power = power * 1.1 end
  if ship.Upgrades[19]=="Yes" then power = power * 1.15 end
  if ship.Upgrades[26]=="Yes" then power = power * 1.2 end
  return math.ceil(power)
end

function getcannonrange(ship)
  local range = ship.Range
  if ship.Upgrades[11]=="Yes" then range = range * 1.1 end
  if ship.Upgrades[18]=="Yes" then range = range * 1.15 end
  if ship.Upgrades[25]=="Yes" then range = range * 1.2 end
  return math.ceil(range)
end

function gethealth(ship)
  return gethull(ship) - ship.Wounds
end

function gethull(ship)
  local hull = ship.Hull
  if ship.Upgrades[7]=="Yes" then hull = hull * 1.1 end
  if ship.Upgrades[15]=="Yes" then hull = hull * 1.15 end
  if ship.Upgrades[22]=="Yes" then hull = hull * 1.2 end
  return math.ceil(hull) 
end

function getmaxspeed(ship)
  local max = ship.MaxSpeed
  if ship.Upgrades[13]=="Yes" then max = max * 1.1 end
  if ship.Upgrades[20]=="Yes" then max = max * 1.15 end
  if ship.Upgrades[27]=="Yes" then max = max * 1.2 end
  return max
end

function gethealthpercent(ship)
  return math.floor((gethealth(ship)*100/gethull(ship))+0.5)
end

function getreloadspeed(ship)
  local reload = 0
  local i
  for i = 1, #ship.Crew, 1 do
    reload = reload + ship.Crew[i].Reload
  end
  if ship.Upgrades[6]=="Yes" then reload = reload*1.05 end
  if ship.Upgrades[14]=="Yes" then reload = reload*1.1 end
  if ship.Upgrades[21]=="Yes" then reload = reload*1.15 end
  return math.ceil(reload)
end

function getrowstrength(ship)
  local str = 0
  local i
  for i = 1, #ship.Crew, 1 do
    str = str + ship.Crew[i].Row
  end
  return math.min(str, 100)/100
end

function getislanddocked(x,y)
  local d = 0
  local i
  for i = 1, #tISLAND, 1 do
    if distancebetween(x, y, tISLAND[i].X, tISLAND[i].Y) <= (tISLAND[i].Size/2)+150 then d = i end
  end
  return d
end

function getislandlocationfree(x,y,size)
  local free = true
  local i
  if next(tISLAND)~=nil then
    for i = 1, #tISLAND, 1 do
      if distancebetween(x,y,tISLAND[i].X,tISLAND[i].Y)<((size+tISLAND[i].Size+600)/2)+tSETTINGS.Spacing then free = false end
    end
  end
  return free
end

function getshiplocationfree(x,y)
  local free = true
  local i
  for i = 1, #tISLAND, 1 do
    if distancebetween(x,y,tISLAND[i].X,tISLAND[i].Y)<(tISLAND[i].Size/2)+60 then free=false end
  end
  if next(tSHIP)~=nil then
    for i = 1, #tSHIP, 1 do
      if distancebetween(x,y,tSHIP[i].X,tSHIP[i].Y)<120 then free=false end
    end
  end
  if distancebetween(x,y,pSHIP.X,pSHIP.Y)<120 then free=false end
  return free
end

function getscoutrange()
  local r = 300
  if gethasrole("Lookout") then r = math.floor(r*1.5) end
  if getisnight() then r = math.floor(r/2) end
  if pSHIP.Upgrades[3]=="Yes" then r = math.ceil(r*1.2) end
  return r
end

function getmaxturn(ship)
  local turn = ship.MaxTurn
  if ship.Upgrades[10]=="Yes" then turn = turn * 1.1 end
  if ship.Upgrades[17]=="Yes" then turn = turn * 1.15 end
  if ship.Upgrades[24]=="Yes" then turn = turn * 1.2 end
  return turn
end

function gethasrole(role)
  local r = false
  local i
  for i = 1, #pSHIP.Crew, 1 do
    if pSHIP.Crew[i].Role==role then r = true end
  end
  return r
end

function gethasrequirement(requirement)
  local gotit = false
  local i
  for i = 1, #tUPGRADES, 1 do
    if tUPGRADES[i].Name==requirement and pSHIP.Upgrades[i]=="Yes" then gotit = true end
  end
  return gotit
end

function getisnight()
  if tCLOCK.Hour<=4 or tCLOCK.Hour>=21 then return true
  else return false end
end

function getamountincargo(goods)
  local amount = 0
  local i
  if next(pSHIP.Cargo)~=nil then
    for i = 1, #pSHIP.Cargo, 1 do
      if pSHIP.Cargo[i]==goods then amount = amount + 1 end
    end
  end
  return amount
end

function removecargo(goods)
  local i = 1
  if next(pSHIP.Cargo)~=nil then
    while pSHIP.Cargo[i]~=goods and i<#pSHIP.Cargo do
      i = i + 1
    end
    table.remove(pSHIP.Cargo, i)
  end
end

function worldtoscreen(x,y)
  return 400+x-gCAMERA_X, 300-y+gCAMERA_Y
end

function getisonscreen(ship)
  local x, y = worldtoscreen(ship.X, ship.Y)
  if x>=-50 and x<=850 and y>=-50 and y<=650 then return true
  else return false end
end

function screentoworld(x,y)
  return (x+gCAMERA_X-400)/gRATIOX, (300-y-gCAMERAY)/gRATIOY
end

function feedcrew()
  local cook, success = 1, true
  local i, j
  if gethasrole("Cook") then cook = 0.8 end
  for i = #pSHIP.Crew, 1, -1 do
    if pSHIP.Food > 0 then pSHIP.Food = math.max(pSHIP.Food-(cook*0.1), 0)
    else
      success = false
      pSHIP.Crew[i].Morale = math.max(pSHIP.Crew[i].Morale-5, 0)
      damagecrewmember(i, math.random(10))
    end
  end
  if success ~= true then addfloatytext("Your crew are hungry and you have no food!") end
end

function damagecrewmember(c, amount)
  local i
  pSHIP.Crew[c].Health = math.max(pSHIP.Crew[c].Health-amount, 0)
  if pSHIP.Crew[c].Health == 0 then
    addfloatytext(pSHIP.Crew[c].Name.." died.")
    if pSHIP.Crew[c].Role=="Captain" then shipwreck(pSHIP)
    else 
      table.remove(pSHIP.Crew, c)
      for i = 1, #pSHIP.Crew, 1 do
        pSHIP.Crew[i].Morale = math.max(pSHIP.Crew[i].Morale - 10, 0)
      end
    end
  end
end

function paycrew()
  local success = true
  local i
  for i = 1, #pSHIP.Crew, 1 do
    if pSHIP.Gold>=pSHIP.Crew[i].Pay then
      pSHIP.Gold = pSHIP.Gold - pSHIP.Crew[i].Pay
    else
      pSHIP.Gold = 0
      pSHIP.Crew[i].Morale = math.max(pSHIP.Crew[i].Morale - 25, 0)
      success = false
    end
    pSHIP.Crew[i].Days = pSHIP.Crew[i].Days + 1
  end
  if #pSHIP.Crew==1 then -- Do nothing, only the captain aboard!
  elseif success == true then 
    addfloatytext("Your crew has been paid.\nYou have "..math.floor(pSHIP.Gold).." gold remaining.")
  else addfloatytext("You cannot afford to pay your crew!\nBe careful, you may face mutiny!") end
end

function killcrew(i)
  table.remove(pSHIP.Crew, i)
end

function collideisland(ship)
  if ship.Invulnerable == 0 then
    damageship(ship, math.floor(gethull(ship)/4))
    if getisonscreen(ship) then playsound(8+math.random(4)) end
  end
end

function collideship(ship)
  if ship.Invulnerable == 0 then
    damageship(ship, math.floor(gethull(ship)/5))
    if getisonscreen(ship) then playsound(8+math.random(4)) end
  end
end

function damageship(ship, amount)
  if ship.Invulnerable == 0 then
    ship.Wounds = ship.Wounds + math.min(amount, gethealth(ship))
    ship.Invulnerable = math.ceil(tSETTINGS.FPS/2)
    if gethealth(ship)==0 then shipwreck(ship) end
  end
end

function shipwreck(ship)
  if getisonscreen(ship) then playsound(14) end
  ship.Version = 5
  ship.Speed = 0
  ship.Z = 0
  ship.Anchor = "Down"
  if ship.Owner == "Player" then 
    gGAMEMODE = "ending"
    gENDING = tSETTINGS.FPS * 4
  end
end

function enterbattle()
  if mBATTLE == false then
    mBATTLE = true
    if tSETTINGS.Music=="Yes" then musicplay(1 + math.random(4)) end
  end
  mCOUNT = tSETTINGS.FPS * 20
end

function spawnbullet(x, y, z, dmg, range, owner)
  local bullet = {}
  bullet.X, bullet.Y, bullet.Z, bullet.Speed = x, y, z, 10
  bullet.Width, bullet.Height = 12, 12
  bullet.Damage, bullet.Count = dmg, range
  bullet.Owner = owner
  table.insert(tBULLET, deepcopy(bullet))
end

function spawnexplosion(x, y)
  local explode = {}
  explode.X, explode.Y = x, y
  explode.Version = 1
  explode.Count = 5
  explode.Refresh = math.ceil(tSETTINGS.FPS/10)
  table.insert(tEXPLODE, deepcopy(explode))
end

function playsound(i, interrupt)  -- Play sound
  interrupt = interrupt or "No"
  if sSOUND[i]~=nil and tSETTINGS.Sound=="Yes" then 
    if interrupt == "Yes" then 
      sSOUND[i]:stop()
      love.audio.play(sSOUND[i])
    else
      if not sSOUND[i]:isPlaying() then love.audio.play(sSOUND[i]) end
    end
  end
end

function playwind(wind)
  wind = wind or "On"
  if wind == "On" and tSETTINGS.Sound=="Yes" and not sWIND:isPlaying() then love.audio.play(sWIND)
  elseif wind == "Off" and sWIND:isPlaying() then love.audio.stop(sWIND)
  else end
  if sWIND:isPlaying() then
    sWIND:setLooping(true)
    sWIND:setVolume((tWIND.Speed^1.5)/512)
  end
end

function playwaves(waves)
  local w = {-5, 5, 15, 25}
  local i
  waves = waves or "On"
  for i = 1, math.min(#w, #sWAVES), 1 do
    if waves == "On" and tSETTINGS.Sound=="Yes" and not sWAVES[i]:isPlaying() then love.audio.play(sWAVES[i])
    elseif waves == "Off" and sWAVES[i]:isPlaying() then love.audio.stop(sWAVES[i])
    else end
    if sWAVES[i]:isPlaying() then
      sWAVES[i]:setLooping(true)
      if tWIND.Speed>=w[i] then sWAVES[i]:setVolume(math.min(tWIND.Speed-w[i],10)/10)
      else sWAVES[i]:setVolume(0) end
    end
  end
end

function processhour()
  local i, j
  tCLOCK.Min = 0
  tCLOCK.Hour = tCLOCK.Hour + 1
  if tCLOCK.Hour==5 then playsound(6) end
  if tCLOCK.Hour==6 then paycrew() end
  if tCLOCK.Hour==6 or tCLOCK.Hour==18 then
    tWIND.Start = math.floor(tWIND.Speed+0.5)
    tWIND.Target = tWIND.Speed+math.random(16)-math.random(16)+math.random(16)-math.random(16)
    tWIND.Target = math.max(math.min(tWIND.Target, 64), 1)
    calculatemaps()
    distributecrew()
  end
  feedcrew()
  if tCLOCK.Hour >= 24 then processday() end
  for i = 1, #tISLAND, 1 do
    tISLAND[i].Forage = math.max(tISLAND[i].Forage-1, 0)
  end
  for i = 1, #pSHIP.Crew, 1 do
    j = 3
    if gethasrole("First Mate") and pSHIP.Docked == 0 then j = j - 1 end
    if getisnight() and pSHIP.Docked == 0 then j = j + 2 end
    pSHIP.Crew[i].Morale = math.max(pSHIP.Crew[i].Morale - (math.random(j)-1), 0)
    if gethasrole("Ship's Surgeon") then damagecrewmember(i, math.random(2)-1)
    else damagecrewmember(i, math.random(3)-1) end
  end
  if gettablesize(pSHIP.Crew)>=2 then checkmorale() end
end

function processday()
  tCLOCK.Hour = 0
  tCLOCK.Day = tCLOCK.Day + 1
end

function forage()
  if tISLAND[pSHIP.Docked].Forage==0 then
    tISLAND[pSHIP.Docked].Forage = 24
    pSHIP.Food = math.min(pSHIP.Food+5, 100)
    processhour()
    processhour()
    addfloatytext("You search for a couple of hours and find some food.")
  else 
    processhour()
    processhour()
    addfloatytext("You search and find nothing.")
  end
  gFADE = 255
end

function checkmorale()
  local m = 0
  local i
  for i = 1, #pSHIP.Crew, 1 do
    m = m + pSHIP.Crew[i].Morale
  end
  m = m / #pSHIP.Crew
  if m<=10 then 
    addfloatytext("Mutiny! You have lost control of your ship.")
    gGAMEMODE = "ending"
    gENDING = tSETTINGS.FPS * 4
  elseif m<=15 then addfloatytext("The morale of your crew is dangerously low. Be careful, they may mutiny!")
  elseif m<=20 then addfloatytext("The morale of your crew is low. They are not happy.")
  else end
end

function getplayerwelcome(i)
  local welcome = true
  if tISLAND[i].Government == "Pirate's Code" and pSHIP.Type ~= 5 then welcome = false
  elseif tISLAND[i].Government == "Naval Court" and pSHIP.Type == 2 then welcome = false
  else end
  return welcome
end

function addfloatytext(text, c, interrupt)
  c = c or tSETTINGS.FPS*6
  interrupt = interrupt or "No"
  local entry = {Text=text, Count=c}
  if interrupt == "Yes" then table.insert(gFLOATTEXT, 1, deepcopy(entry))
  else table.insert(gFLOATTEXT, deepcopy(entry)) end
end